@@ -2,8 +2,8 @@
 /*                                                                            */
 /*     Copyright (C)    2000-2008 Cédric Auger (cedric@grisbi.org)            */
 /*          2003-2008 Benjamin Drieu (bdrieu@april.org)                       */
-/*                      2008-2018 Pierre Biava (grisbi@pierre.biava.name)     */
-/*          http://www.grisbi.org                                             */
+/*                      2008-2020 Pierre Biava (grisbi@pierre.biava.name)     */
+/*          https://www.grisbi.org/                                           */
 /*                                                                            */
 /*  This program is free software; you can redistribute it and/or modify      */
 /*  it under the terms of the GNU General Public License as published by      */
@@ -33,6 +33,7 @@
 #include "grisbi_app.h"
 #include "parametres.h"
 #include "structures.h"
+#include "erreur.h"
 /*END_INCLUDE*/
 
 /*START_STATIC*/
@@ -41,61 +42,49 @@
 /*START_EXTERN*/
 /*END_EXTERN*/
 
-/** All messages */
-struct ConditionalMessage messages[] =
+/** All delete messages */
+ConditionalMsg tab_delete_msg[NBRE_MSG_DELETE] =
 {
-    { "ofx-security-not-implemented", N_("Security feature not implemented"),
-      N_("This file contains security information, which processing is not implemented at "
-      "this moment."),
+    { "delete-child-transaction", N_("Delete a child transaction."),
+      NULL,
       FALSE, FALSE, },
 
-    { "encryption-is-irreversible", N_("Encryption is irreversible."),
-      N_("Grisbi encrypts files in a very secure way that does not allow recovery without "
-      "original password. It means that if you forget your password, you will lose all "
-      "your data. Use with caution.\n\nI repeat: if you ever forget your password, there "
-      "is no coming back, we cannot help you."),
+    { "delete-transaction",  N_("Delete a transaction."),
+      NULL,
       FALSE, FALSE, },
 
-    { "account-file-readable",  N_("Account file is world readable."),
-      N_("Your account file should not be readable by anybody else, but it is. You should "
-      "change its permissions."),
+    { "delete-child-scheduled", N_("Delete a child of scheduled transaction."),
+      NULL,
       FALSE, FALSE, },
 
+    { "delete-scheduled", N_("Delete a scheduled transaction."),
+      NULL,
+      FALSE, FALSE, },
+
+    { "delete-scheduled-occurrences", N_("Delete one or all occurrences of scheduled "
+      "transaction."),
+      NULL,
+      FALSE, FALSE, },
+
+    { "delete-rule",  N_("Delete a rule file import."),
+      NULL,
+      FALSE, FALSE, },
+
+    { NULL, NULL, NULL, FALSE, FALSE },
+};
+
+/** All messages */
+ConditionalMsg tab_warning_msg[NBRE_MSG_WARNINGS] =
+{
     { "account-already-opened", N_("File \"%s\" is already opened"),
       N_("Either this file is already opened by another user or it wasn't closed correctly "
       "(maybe Grisbi crashed?).\nGrisbi can't save the file unless you activate the "
       "\"Force saving locked files\" option in setup."),
       FALSE, FALSE, },
 
-    { "minimum-balance-alert", N_("Account under desired balance"),
-      N_("Grisbi detected that an account is under a desired balance: %s"),
-      FALSE, FALSE, },
-
-    { "no-budgetary-line", N_("No budgetary line was entered"),
-      N_("This transaction has no budgetary line entered.  You should use them to "
-      "easily produce budgets and make reports on them."),
-      FALSE, FALSE, },
-
-    { "recover-split", N_("Recover split?"),
-      N_("This is a split of transaction, associated transactions can be recovered as "
-      "in last transaction with this payee.  Do you want to recover them?"),
-      FALSE, FALSE, },
-
-    { "no-inconsistency-found", N_("No inconsistency found."),
-      N_("Grisbi found no known inconsistency in accounts processed."),
-      FALSE, FALSE, },
-
-    { "reconcile-transaction", N_("Confirmation of manual (un)reconciliation"),
-      N_("You are trying to reconcile or unreconcile a transaction manually, "
-	  "which is not a recommended action.\n"
-      "Are you really sure you know what you are doing?"),
-      FALSE, FALSE, },
-
-    { "reconcile-start-end-dates", N_("Reconcile start and end dates."),
-      N_("In previous versions, Grisbi did not save start date, end date and balance for "
-      "reconciliation. This is now done, so Grisbi will try to guess values from your "
-      "accounts. Thought this can not harm data coherence, false values can be guessed. "
-      "Please check in the Preferences window for more information."),
+    { "account-file-readable",  N_("Account file is world readable."),
+      N_("Your account file should not be readable by anybody else, but it is. You should "
+      "change its permissions."),
       FALSE, FALSE, },
 
     { "development-version", N_("You are running Grisbi version %s"),
@@ -104,16 +93,38 @@ struct ConditionalMessage messages[] =
       "Make a copy now."),
       FALSE, FALSE },
 
-/*
-    { "", N_(),
-      N_(),
+    { "encryption-is-irreversible", N_("Encryption is irreversible."),
+      N_("Grisbi encrypts files in a very secure way that does not allow recovery without "
+      "original password. It means that if you forget your password, you will lose all "
+      "your data. Use with caution.\n\nI repeat: if you ever forget your password, there "
+      "is no coming back, we cannot help you."),
       FALSE, FALSE, },
-*/
+
+    { "minimum-balance-alert", N_("Account under desired balance"),
+      N_("Grisbi detected that an account is under a desired balance: %s"),
+      FALSE, FALSE, },
+
+    { "reconcile-transaction", N_("Confirmation of manual reconciliation"),
+      N_("You are trying to reconcile a transaction manually, "
+	  "which is not a recommended action.\n"
+      "Are you really sure you know what you are doing?"),
+      FALSE, FALSE, },
+
+    { "unreconcile-transaction", N_("Confirmation of manual unreconciliation"),
+      N_("You are trying to unreconcile a transaction manually, "
+	  "which is not a recommended action.\n"
+      "Are you really sure you know what you are doing?"),
+      FALSE, FALSE, },
+
+    { "remove-backup-files", N_("Removing backups from the account file"),
+      N_("You will delete %d backups from your account file that are older than %s."),
+      FALSE, FALSE, },
+
     { NULL, NULL, NULL, FALSE, FALSE },
 };
 
 /******************************************************************************/
-/* Private functions                                                            */
+/* Private functions                                                          */
 /******************************************************************************/
 /**
  * Concatenate two strings and use pango layout to produce a string
@@ -125,7 +136,7 @@ struct ConditionalMessage messages[] =
  * \return		a pango formated string with the two arguments concatenated.
  * 				It returns a newly allocated string which must be freed when no more used.
  **/
-static gchar *make_hint (const gchar *hint,
+static gchar *dialogue_make_hint (const gchar *hint,
 						 const gchar *text)
 {
     gchar *tmp_str = NULL;
@@ -163,6 +174,9 @@ static void dialogue_special (GtkMessageType param,
 
     primary_text = hint ? hint : text;
 
+    if (NULL == grisbi_app_get_active_window (NULL))
+        return;
+
     dialog = gtk_message_dialog_new (GTK_WINDOW (grisbi_app_get_active_window (NULL)),
 									 GTK_DIALOG_DESTROY_WITH_PARENT,
 									 param, GTK_BUTTONS_CLOSE,
@@ -170,7 +184,7 @@ static void dialogue_special (GtkMessageType param,
 
     if (hint)
     {
-        gtk_message_dialog_set_markup (GTK_MESSAGE_DIALOG (dialog), make_hint (primary_text, NULL));
+        gtk_message_dialog_set_markup (GTK_MESSAGE_DIALOG (dialog), dialogue_make_hint (primary_text, NULL));
         gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog), "%s", text);
     }
     else
@@ -194,9 +208,9 @@ static void dialogue_special (GtkMessageType param,
 static gboolean dialogue_update_var (GtkWidget *checkbox,
 									 gint message)
 {
-    messages[message].hidden = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(checkbox));
+    tab_warning_msg[message].hidden = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(checkbox));
 
-    return FALSE;
+	return FALSE;
 }
 
 /**
@@ -220,16 +234,17 @@ static GtkDialog *dialogue_conditional_new (const gchar *text,
     GtkWidget *checkbox;
     GtkWidget *dialog = NULL;
     GtkWidget *vbox;
+	gchar *label;
     gint i;
 
-    if (!var || !strlen (var))
+	if (!var || !strlen (var))
         return NULL;
 
-    for  (i = 0; messages[i].name; i++)
+    for  (i = 0; tab_warning_msg[i].name; i++)
     {
-        if (!strcmp (messages[i].name, var))
+        if (!strcmp (tab_warning_msg[i].name, var))
         {
-            if (messages[i].hidden)
+            if (tab_warning_msg[i].hidden)
             {
                 return NULL;
             }
@@ -246,7 +261,20 @@ static GtkDialog *dialogue_conditional_new (const gchar *text,
 
     vbox = gtk_dialog_get_content_area (GTK_DIALOG (dialog));
 
-    checkbox = gtk_check_button_new_with_label (_("Keep this choice permanently?"));
+	switch (buttons)
+	{
+		case GTK_BUTTONS_NONE:
+		case GTK_BUTTONS_OK:
+		case GTK_BUTTONS_CANCEL:
+		case GTK_BUTTONS_CLOSE:
+			label = _("Do not show this message again");
+			break;
+		case GTK_BUTTONS_YES_NO:
+		case GTK_BUTTONS_OK_CANCEL:
+			label = _("Keep this choice and no longer see this message?");
+			break;
+	}
+    checkbox = gtk_check_button_new_with_label (label);
     g_signal_connect (G_OBJECT (checkbox),
 					  "toggled",
 					  G_CALLBACK (dialogue_update_var),
@@ -423,8 +451,8 @@ GtkWidget *dialogue_special_no_run (GtkMessageType param,
 
     if (hint)
     {
-        gtk_message_dialog_set_markup (GTK_MESSAGE_DIALOG (dialog), make_hint (primary_text, NULL));
-        gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog), "%s", text);
+        gtk_message_dialog_set_markup (GTK_MESSAGE_DIALOG (dialog), dialogue_make_hint (primary_text, NULL));
+        gtk_message_dialog_format_secondary_markup (GTK_MESSAGE_DIALOG (dialog), "%s", text);
     }
     else
          gtk_message_dialog_set_markup (GTK_MESSAGE_DIALOG (dialog), primary_text);
@@ -443,9 +471,9 @@ GtkWidget *dialogue_special_no_run (GtkMessageType param,
  * \return      	FALSE.
  **/
 gboolean dialogue_update_struct_message (GtkWidget *checkbox,
-										 struct ConditionalMessage *message)
+										 ConditionalMsg *msg)
 {
-    message->hidden = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbox));
+    msg->hidden = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbox));
 
     return FALSE;
 }
@@ -468,7 +496,7 @@ void dialogue_conditional_hint (const gchar *text,
 								const gchar *hint,
 								const gchar *var)
 {
-    dialogue_conditional (make_hint(hint, text), var);
+    dialogue_conditional (dialogue_make_hint(hint, text), var);
 }
 
 /**
@@ -482,7 +510,7 @@ void dialogue_conditional_hint (const gchar *text,
  *
  * \return 					TRUE if user pressed 'YES'. FALSE otherwise.
  **/
-gboolean question_yes_no (const gchar *text,
+gboolean dialogue_yes_no (const gchar *text,
 						  const gchar *hint,
 						  gint default_answer)
 {
@@ -496,7 +524,7 @@ gboolean question_yes_no (const gchar *text,
 									 GTK_MESSAGE_QUESTION,
 									 GTK_BUTTONS_YES_NO,
 									 NULL);
-    gtk_message_dialog_set_markup (GTK_MESSAGE_DIALOG (dialog), primary_text);
+    gtk_message_dialog_set_markup (GTK_MESSAGE_DIALOG (dialog), dialogue_make_hint (primary_text, NULL));
 
     if (hint)
         gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog), "%s", text);
@@ -523,38 +551,39 @@ gboolean question_yes_no (const gchar *text,
  *
  * \return TRUE if user pressed 'YES'.  FALSE otherwise.
  */
-gboolean question_conditional_yes_no (const gchar *var)
+gboolean dialogue_conditional_yes_no (const gchar *var)
 {
     GtkDialog *dialog;
     gchar *text;
     gint i;
     gint response;
 
-    for  (i = 0; messages[i].name; i++)
+    for  (i = 0; tab_warning_msg[i].name; i++)
     {
-        if (!strcmp (messages[i].name, var))
+        if (!strcmp (tab_warning_msg[i].name, var))
         {
-            if (messages[i].hidden)
+            if (tab_warning_msg[i].hidden)
             {
-                return messages[i].default_answer;
+                return tab_warning_msg[i].default_answer;
             }
             break;
         }
     }
 
-	text = make_hint (messages[i].hint, messages[i].message);
+	text = dialogue_make_hint (gettext (tab_warning_msg[i].hint), gettext (tab_warning_msg[i].message));
     dialog = dialogue_conditional_new (text, var, GTK_MESSAGE_WARNING, GTK_BUTTONS_YES_NO);
 	g_free (text);
 
     response = gtk_dialog_run (GTK_DIALOG (dialog));
 
     if (response == GTK_RESPONSE_YES)
-        messages[i].default_answer = TRUE;
+        tab_warning_msg[i].default_answer = TRUE;
     else
-        messages[i].default_answer = FALSE;
+        tab_warning_msg[i].default_answer = FALSE;
 
     gtk_widget_destroy (GTK_WIDGET (dialog));
-    return messages[i].default_answer;
+
+	return tab_warning_msg[i].default_answer;
 }
 
 /**
@@ -566,18 +595,40 @@ gboolean question_conditional_yes_no (const gchar *var)
  *
  * \return	TRUE if user pressed 'YES'. FALSE otherwise.
  **/
-gboolean question_conditional_yes_no_with_struct (struct ConditionalMessage *message)
+gboolean dialogue_conditional_yes_no_with_items (const gchar *tab_name,
+												 const gchar *struct_name,
+												 const gchar *tmp_msg)
 {
     GtkWidget *checkbox;
     GtkWidget *dialog;
     GtkWidget *vbox;
+	gint msg_no = 0;
     gchar *text;
     gint response;
 
-    if (message->hidden)
-        return message->default_answer;
+	if (g_strcmp0 (tab_name, "tab_delete_msg") == 0)
+	{
+		for  (msg_no = 0; tab_delete_msg[msg_no].name; msg_no++)
+		{
+			if (strcmp (tab_delete_msg[msg_no].name, struct_name) == 0)
+			{
+				break;
+			}
+		}
+	}
+	else
+	{
+		for  (msg_no = 0; tab_warning_msg[msg_no].name; msg_no++)
+		{
+			if (strcmp (tab_warning_msg[msg_no].name, struct_name) == 0)
+				break;
+		}
+	}
 
-    text = make_hint (gettext (message->hint), message->message);
+    if (tab_delete_msg[msg_no].hidden)
+        return tab_delete_msg[msg_no].default_answer;
+
+    text = dialogue_make_hint (gettext (tab_delete_msg[msg_no].hint), tmp_msg);
     dialog = gtk_message_dialog_new (GTK_WINDOW (grisbi_app_get_active_window (NULL)),
 									 GTK_DIALOG_DESTROY_WITH_PARENT,
 									 GTK_MESSAGE_WARNING,
@@ -589,23 +640,76 @@ gboolean question_conditional_yes_no_with_struct (struct ConditionalMessage *mes
 
     vbox = gtk_dialog_get_content_area (GTK_DIALOG (dialog));
 
-    checkbox = gtk_check_button_new_with_label (_("Do not show this message again"));
+    checkbox = gtk_check_button_new_with_label (_("Keep this choice and no longer see this message?"));
     g_signal_connect (G_OBJECT (checkbox),
 					  "toggled",
                       G_CALLBACK (dialogue_update_struct_message),
-                      message);
+                      &tab_delete_msg[msg_no]);
     gtk_box_pack_start (GTK_BOX (vbox), checkbox, TRUE, TRUE, MARGIN_BOX);
     gtk_widget_show_all (checkbox);
 
     response = gtk_dialog_run (GTK_DIALOG (dialog));
 
     if (response == GTK_RESPONSE_YES)
-        message->default_answer = TRUE;
-    else
-        message->default_answer = FALSE;
+        tab_delete_msg[msg_no].default_answer = TRUE;
+	else
+		tab_delete_msg[msg_no].default_answer = FALSE;
 
     gtk_widget_destroy (GTK_WIDGET (dialog));
-    return message->default_answer;
+
+	return tab_delete_msg[msg_no].default_answer;
+}
+
+/**
+ * Pop up a warning dialog window with a question and a checkbox that allow
+ * this message not to be displayed again thanks to preferences and wait
+ * for user to press 'YES' or 'NO'.
+ *
+ * \param 	struct ConditionalMessage
+ *
+ * \return	TRUE if user pressed 'YES'. FALSE otherwise.
+ **/
+gboolean dialogue_conditional_yes_no_with_struct (ConditionalMsg *msg)
+{
+    GtkWidget *checkbox;
+    GtkWidget *dialog;
+    GtkWidget *vbox;
+    gchar *text;
+    gint response;
+
+    if (msg->hidden)
+        return msg->default_answer;
+
+    text = dialogue_make_hint (gettext (msg->hint), msg->message);
+    dialog = gtk_message_dialog_new (GTK_WINDOW (grisbi_app_get_active_window (NULL)),
+									 GTK_DIALOG_DESTROY_WITH_PARENT,
+									 GTK_MESSAGE_WARNING,
+									 GTK_BUTTONS_YES_NO,
+									 NULL);
+    gtk_dialog_set_default_response (GTK_DIALOG(dialog), GTK_RESPONSE_NO);
+    gtk_message_dialog_set_markup (GTK_MESSAGE_DIALOG (dialog), text);
+	g_free (text);
+
+    vbox = gtk_dialog_get_content_area (GTK_DIALOG (dialog));
+
+    checkbox = gtk_check_button_new_with_label (_("Keep this choice and no longer see this message?"));
+    g_signal_connect (G_OBJECT (checkbox),
+					  "toggled",
+                      G_CALLBACK (dialogue_update_struct_message),
+                      msg);
+    gtk_box_pack_start (GTK_BOX (vbox), checkbox, TRUE, TRUE, MARGIN_BOX);
+    gtk_widget_show_all (checkbox);
+
+    response = gtk_dialog_run (GTK_DIALOG (dialog));
+
+    if (response == GTK_RESPONSE_YES)
+        msg->default_answer = TRUE;
+    else
+        msg->default_answer = FALSE;
+
+    gtk_widget_destroy (GTK_WIDGET (dialog));
+
+	return msg->default_answer;
 }
 
 /**
@@ -616,7 +720,7 @@ gboolean question_conditional_yes_no_with_struct (struct ConditionalMessage *mes
  *
  * \return	message number or -1 is not present.
  */
-gint question_conditional_yes_no_get_no_struct (struct ConditionalMessage *msg,
+gint dialogue_conditional_yes_no_get_no_struct (ConditionalMsg *msg,
 												const gchar *name)
 {
     gint i;
@@ -637,7 +741,7 @@ gint question_conditional_yes_no_get_no_struct (struct ConditionalMessage *msg,
  * \return	a pango formated string It returns a newly allocated string which must
  *			be freed when no more used.
  **/
-gchar *make_red (const gchar *text)
+gchar *dialogue_make_red (const gchar *text)
 {
     gchar *tmp_str;
 
@@ -655,7 +759,7 @@ gchar *make_red (const gchar *text)
  * \return			a pango formated string It returns a newly allocated string which must
  *					be freed when no more used.
  */
-gchar *make_pango_attribut (const gchar *attribut,
+gchar *dialogue_make_pango_attribut (const gchar *attribut,
 							const gchar *text)
 {
     gchar *span_format;
@@ -676,7 +780,7 @@ gchar *make_pango_attribut (const gchar *attribut,
  * \return a pango formated string It returns a newly allocated string which must
  *         be freed when no more used.
  **/
-gchar *make_blue (const gchar *text)
+gchar *dialogue_make_blue (const gchar *text)
 {
     gchar *tmp_str;
 
@@ -726,25 +830,25 @@ void dialogue_error_memory (void)
  *
  * \return
  **/
-void dialog_message (const gchar *label, ...)
+void dialogue_message (const gchar *label, ...)
 {
     va_list ap;
     gint i = 0;
 
-    while (messages[i].name)
+    while (tab_warning_msg[i].name)
     {
-		if (!strcmp (messages[i].name, label))
+		if (!strcmp (tab_warning_msg[i].name, label))
 		{
-			if (! messages[i].hidden)
+			if (! tab_warning_msg[i].hidden)
 			{
 				gchar hint_buffer[1024];
 				gchar message_buffer[1024];
 
 				va_start (ap, label);
-				vsnprintf (hint_buffer, sizeof hint_buffer, _(messages[i].hint), ap);
-				vsnprintf (message_buffer, sizeof message_buffer, _(messages[i].message), ap);
+				vsnprintf (hint_buffer, sizeof hint_buffer, _(tab_warning_msg[i].hint), ap);
+				vsnprintf (message_buffer, sizeof message_buffer, _(tab_warning_msg[i].message), ap);
 
-				dialogue_conditional_hint (message_buffer,hint_buffer, messages[i].name);
+				dialogue_conditional_hint (message_buffer,hint_buffer, tab_warning_msg[i].name);
 				va_end(ap);
 			}
 			return;
@@ -817,6 +921,30 @@ GtkWidget *dialog_get_content_area (GtkWidget *dialog)
     return gtk_dialog_get_content_area (GTK_DIALOG (dialog));
 }
 
+/**
+ *
+ *
+ * \param
+ *
+ * \return
+ **/
+gpointer dialogue_get_tab_delete_msg (void)
+{
+	return &tab_delete_msg[0];
+}
+
+/**
+ *
+ *
+ * \param
+ *
+ * \return
+ **/
+gpointer dialogue_get_tab_warning_msg (void)
+{
+	return &tab_warning_msg[0];
+}
+
 /**
  *
  *
