@@ -4,7 +4,7 @@
 /*                                                                            */
 /*     Copyright (C)    2000-2008 Cédric Auger (cedric@grisbi.org)            */
 /*          2003-2008 Benjamin Drieu (bdrieu@april.org)                       */
-/*          http://www.grisbi.org                                             */
+/*          https://www.grisbi.org/                                            */
 /*                                                                            */
 /*  This program is free software; you can redistribute it and/or modify      */
 /*  it under the terms of the GNU General Public License as published by      */
@@ -56,17 +56,18 @@
 
 
 /*START_STATIC*/
-/*END_STATIC*/
-
-/*START_EXTERN*/
-/*END_EXTERN*/
-
+static gboolean debug_mode = FALSE;
 static gint debugging_grisbi;
 
 /* path and name of the file containing the log when debug mode is active
  * this values should not be freed when begin a new file to continue the log */
 static FILE *debug_file = NULL;
 
+/*END_STATIC*/
+
+/*START_EXTERN*/
+/*END_EXTERN*/
+
  /**
  * return a string with the current time
  * use to debug lines
@@ -102,15 +103,18 @@ static gchar *debug_get_debug_time ( void )
  **/
 static GtkWidget *debug_print_backtrace ( void )
 {
-    void *backtrace_content[15];
+    void *backtrace_content[50];
     int backtrace_size;
     gint i;
     gchar **backtrace_strings, *text = g_strdup("");
     GtkWidget * label;
 
-    backtrace_size = backtrace (backtrace_content, 15);
+    backtrace_size = backtrace (backtrace_content, 50);
     backtrace_strings = backtrace_symbols (backtrace_content, backtrace_size);
 
+	text = g_strconcat(text, "Grisbi version: " VERSION "\n", NULL);
+	g_print ("%s", text);
+
     g_print ("%s : %d elements in stack.\n", debug_get_debug_time(), backtrace_size);
 
     for (i = 0; i < backtrace_size; i++)
@@ -143,40 +147,47 @@ void debug_traitement_sigsegv ( gint signal_nb )
 	const gchar *filename;
     gchar *errmsg = g_strdup ( "" );
 	gchar *old_errmsg;
+    const gchar *signal_name;
     gchar *tmp_str;
+	gboolean file_is_loading;
 #ifdef HAVE_BACKTRACE
     GtkWidget *expander;
+    GtkWidget *scrolled_window;
 #endif
 
 	switch ( signal_nb )
 	{
 		case 2:
-			tmp_str = "SIGINT";
+			signal_name = "SIGINT";
 			break;
 		case 11:
-			tmp_str = "SIGSEGV";
+			signal_name = "SIGSEGV";
 			break;
 		case 15:
-			tmp_str = "SIGTERM";
+			signal_name = "SIGTERM";
 			break;
 		default:
-			tmp_str = "NA";
+			signal_name = "NA";
 			printf ("signal number: %d\n", signal_nb );
 	}
 
-	printf ("signal name = %s\n", tmp_str );
+	printf ("\nsignal name = %s\n", signal_name );
 
 	/* on récupère le nom du fichier si possible */
 	filename = grisbi_win_get_filename (NULL);
+	if (filename)
+		file_is_loading = TRUE;
+	else
+		file_is_loading = FALSE;
 
     /*   il y a 3 possibilités : */
     /*     soit on était en train de charger un fichier, c'est que celui-ci est corrompu */
     /* soit on était en train de sauver un fichier, et là on peut rien faire */
     /* sinon on essaie de sauver le fichier sous le nom entouré de # */
 
-    if ( run.file_is_loading || run.file_is_saving || !gsb_file_get_modified ())
+    if (file_is_loading || run.file_is_saving || !gsb_file_get_modified ())
     {
-		if ( run.file_is_loading )
+		if (file_is_loading )
 		{
 			old_errmsg = errmsg;
 			errmsg = g_strconcat ( errmsg, _("File is corrupted."), NULL );
@@ -211,7 +222,7 @@ void debug_traitement_sigsegv ( gint signal_nb )
 
 		grisbi_win_status_bar_message ( _("Save file") );
 
-		gsb_file_save_save_file ( nom_fichier_comptes, conf.compress_file, FALSE );
+		gsb_file_save_save_file (nom_fichier_comptes, FALSE, 0);
 
 		grisbi_win_status_bar_clear();
 
@@ -227,7 +238,7 @@ void debug_traitement_sigsegv ( gint signal_nb )
     old_errmsg = errmsg;
     errmsg = g_strconcat ( errmsg,
 			   "\n\n",
-			   _("Please report this problem to <tt>http://www.grisbi.org/bugsreports/</tt>.  "),
+			   _("Please report this problem to <a href=\"https://www.grisbi.org/bugsreports/\">https://www.grisbi.org/bugsreports/</a>.\n"),
 			   NULL );
      g_free ( old_errmsg );
 
@@ -250,8 +261,12 @@ void debug_traitement_sigsegv ( gint signal_nb )
     g_free ( tmp_str );
 
     gtk_expander_set_use_markup ( GTK_EXPANDER ( expander ), TRUE );
+    gtk_expander_set_expanded( GTK_EXPANDER ( expander ), TRUE );
     gtk_container_add ( GTK_CONTAINER ( expander ), debug_print_backtrace() );
-    gtk_box_pack_start ( GTK_BOX ( dialog_get_content_area ( dialog ) ), expander, FALSE, FALSE, 6 );
+    scrolled_window = gtk_scrolled_window_new (NULL, NULL);
+    gtk_scrolled_window_set_min_content_height(GTK_SCROLLED_WINDOW(scrolled_window), 400);
+    gtk_container_add (GTK_CONTAINER (scrolled_window), expander);
+    gtk_box_pack_start ( GTK_BOX ( dialog_get_content_area ( dialog ) ), scrolled_window, FALSE, FALSE, 6 );
 
     gtk_widget_show_all ( dialog );
 #endif
@@ -279,65 +294,51 @@ void debug_initialize_debugging ( gint level )
 	gchar* tmp_str1;
 	gchar* tmp_str2;
 
+    /* on choppe la variable d'environnement en priorité */
 	if (getenv ("DEBUG_GRISBI"))
-    {
-        /* on choppe la variable d'environnement */
         debug_variable = utils_str_atoi (getenv ("DEBUG_GRISBI"));
+	else
+		debug_variable = level;
 
-        /* on verifie que la variable est cohérente */
-        if (debug_variable > 0 && debug_variable <= MAX_DEBUG_LEVEL)
-        {
-            /* on renseigne le texte du level de debug */
-            debugging_grisbi = debug_variable;
-            switch(debug_variable)
-            {
-            case DEBUG_LEVEL_ALERT: { debug_level="Alert"; break; }
-            case DEBUG_LEVEL_IMPORTANT: { debug_level="Important"; break; }
-            case DEBUG_LEVEL_NOTICE: { debug_level="Notice"; break; }
-            case DEBUG_LEVEL_INFO: { debug_level="Info"; break; }
-            case DEBUG_LEVEL_DEBUG: { debug_level="Debug"; break; }
-            }
+	/* on verifie que la variable est cohérente sinon on se met en mode debug max */
+	if (debug_variable > 0 && debug_variable <= MAX_DEBUG_LEVEL)
+	{
+		/* on renseigne le texte du level de debug */
+		debugging_grisbi = debug_variable;
+		switch(debug_variable)
+		{
+		case DEBUG_LEVEL_ALERT: { debug_level="Alert"; break; }
+		case DEBUG_LEVEL_IMPORTANT: { debug_level="Important"; break; }
+		case DEBUG_LEVEL_NOTICE: { debug_level="Notice"; break; }
+		case DEBUG_LEVEL_INFO: { debug_level="Info"; break; }
+		case DEBUG_LEVEL_DEBUG: { debug_level="Debug"; break; }
+		}
 
-            /* on affiche un message de debug pour indiquer que le debug est actif */
-            tmp_str1 = g_strdup_printf ( _("GRISBI %s Debug"), VERSION );
-            tmp_str2 = g_strdup_printf ( _("Debug enabled, level is '%s'"),debug_level);
-			g_print ("%s, %2f : %s - %s\n",
-					 debug_get_debug_time (),
-					 (clock() + 0.0)/ CLOCKS_PER_SEC,
-					 tmp_str1,
-					 tmp_str2);
-            g_free ( tmp_str1 );
-            g_free ( tmp_str2 );
-        }
-        else
-        {
-            gchar* tmp_str;
+		/* on affiche un message de debug pour indiquer que le debug est actif */
+		tmp_str1 = g_strdup_printf ( _("GRISBI %s Debug"), VERSION );
+		tmp_str2 = g_strdup_printf ( _("Debug enabled, level is '%s'"),debug_level);
+		g_print ("%s, %2f : %s - %s\n",
+				 debug_get_debug_time (),
+				 (clock() + 0.0)/ CLOCKS_PER_SEC,
+				 tmp_str1,
+				 tmp_str2);
+		g_free ( tmp_str1 );
+		g_free ( tmp_str2 );
+	}
+	else
+	{
+		gchar* tmp_str;
 
-            debugging_grisbi = DEBUG_LEVEL_DEBUG;
-            /* on affiche un message de debug pour indiquer que le debug est actif */
-            tmp_str = g_strdup_printf (_("GRISBI %s Debug"),VERSION);
-			g_print ("%s, %2f : %s - initialize_debugging - %s\n",
-					 debug_get_debug_time (),
-					 (clock() + 0.0)/ CLOCKS_PER_SEC,
-					 tmp_str,
-					 _("Default debug level: Debug"));
-            g_free (tmp_str);
-        }
-    }
-    else
-    {
-        gchar* tmp_str;
-
-        debugging_grisbi = DEBUG_LEVEL_DEBUG;
-        /* on affiche un message de debug pour indiquer que le debug est actif */
-        tmp_str = g_strdup_printf (_("GRISBI %s Debug"),VERSION);
+		debugging_grisbi = DEBUG_LEVEL_DEBUG;
+		/* on affiche un message de debug pour indiquer que le debug est actif */
+		tmp_str = g_strdup_printf (_("GRISBI %s Debug"),VERSION);
 		g_print ("%s, %2f : %s - initialize_debugging - %s\n",
 				 debug_get_debug_time (),
 				 (clock() + 0.0)/ CLOCKS_PER_SEC,
 				 tmp_str,
 				 _("Default debug level: Debug"));
-        g_free (tmp_str);
-    }
+		g_free (tmp_str);
+	}
 }
 
 /**
@@ -406,7 +407,7 @@ void debug_message_string ( const gchar *prefixe,
 {
 
 	/* il faut bien entendu que le mode debug soit actif ou que l'on force l'affichage */
-    if (debugging_grisbi || force_debug_display || conf.debug_mode)
+    if (level <= debugging_grisbi || force_debug_display || debug_mode)
     {
         gchar* tmp_str;
 
@@ -420,7 +421,7 @@ void debug_message_string ( const gchar *prefixe,
                         debug_get_debug_time (), (clock() + 0.0)/ CLOCKS_PER_SEC, prefixe,
                         file, line, function);
 
-        if ( conf.debug_mode )
+        if ( debug_mode && debug_file)
         {
             fwrite ( tmp_str, sizeof (gchar), strlen ( tmp_str ), debug_file );
             fflush ( debug_file );
@@ -455,25 +456,10 @@ void debug_message_int ( const gchar *prefixe,
                         gint level,
                         gboolean force_debug_display )
 {
-	/* il faut bien entendu que le mode debug soit actif ou que l'on force l'affichage */
-    if ( ( debugging_grisbi && level <= debugging_grisbi) || force_debug_display || conf.debug_mode )
-    {
-        gchar* tmp_str;
-
-        /* on affiche dans la console le message */
-        tmp_str = g_strdup_printf(_("%s, %2f : %s - %s:%d:%s - %d\n"),
-                        debug_get_debug_time (), (clock() + 0.0)/ CLOCKS_PER_SEC, prefixe,
-                        file, line, function, message);
-
-        if (conf.debug_mode)
-        {
-            fwrite ( tmp_str, sizeof (gchar), strlen ( tmp_str ), debug_file );
-            fflush ( debug_file );
-        }
-
-        g_print( "%s", tmp_str );
-        g_free ( tmp_str );
-    }
+	gchar* tmp_str;
+	tmp_str = g_strdup_printf("%d", message);
+	debug_message_string(prefixe, file, line, function, tmp_str, level, force_debug_display);
+	g_free ( tmp_str );
 }
 
 
@@ -501,25 +487,34 @@ void debug_message_real ( const gchar *prefixe,
                         gint level,
                         gboolean force_debug_display )
 {
-	/* il faut bien entendu que le mode debug soit actif ou que l'on force l'affichage */
-    if ( ( debugging_grisbi && level <= debugging_grisbi) || force_debug_display || conf.debug_mode )
-    {
-        gchar* tmp_str;
+	gchar* tmp_str;
+	tmp_str = g_strdup_printf("%"G_GINT64_MODIFIER"d E %d", message.mantissa, message.exponent);
+	debug_message_string(prefixe, file, line, function, tmp_str, level, force_debug_display);
+	g_free ( tmp_str );
+}
 
-        /* on affiche dans la console le message */
-        tmp_str = g_strdup_printf ("%s, %2f : %s - %s:%d:%s - %"G_GINT64_MODIFIER"d E %d\n",
-                        debug_get_debug_time (), (clock() + 0.0)/ CLOCKS_PER_SEC, prefixe,
-                        file, line, function, message.mantissa, message.exponent );
+void debug_message_ptr ( const gchar *prefixe,
+                        const gchar *file,
+                        gint line,
+                        const char *function,
+                        void * message,
+                        gint level,
+                        gboolean force_debug_display )
+{
+	gchar* tmp_str;
+	tmp_str = g_strdup_printf("%p", message);
+	debug_message_string(prefixe, file, line, function, tmp_str, level, force_debug_display);
+	g_free ( tmp_str );
+}
 
-        if ( conf.debug_mode )
-        {
-            fwrite ( tmp_str, sizeof (gchar), strlen ( tmp_str ), debug_file );
-            fflush ( debug_file );
-        }
-
-        g_print( "%s", tmp_str );
-        g_free ( tmp_str );
-    }
+/**
+* Enable logs and send them to a file
+* Can be used when grisbi is NOT started from a terminal
+* */
+void debug_start_log_file(void)
+{
+    debug_file = utils_files_utf8_fopen ( "/tmp/grisbi_log.txt", "w" );
+	debug_mode = TRUE;
 }
 
 /**
@@ -577,7 +572,7 @@ gboolean debug_start_log ( void )
     {
         gchar *tmp_str_2;
 
-		conf.debug_mode = TRUE;
+		debug_mode = TRUE;
         /* début du mode de débogage */
         tmp_str = g_strdup_printf(_("%s, %2f : Debug - %s:%d:%s\n\n"),
                         debug_get_debug_time ( ),
@@ -624,8 +619,11 @@ gboolean debug_start_log ( void )
         tmp_str = gsb_ImportFormats_get_list_formats_to_string ( );
         tmp_str_2 = g_strconcat ( tmp_str, "\n", NULL );
 
-        fwrite ( tmp_str_2, sizeof (gchar), strlen ( tmp_str_2 ), debug_file );
-	    fflush ( debug_file );
+        if (tmp_str_2)
+        {
+            fwrite ( tmp_str_2, sizeof (gchar), strlen ( tmp_str_2 ), debug_file );
+            fflush ( debug_file );
+        }
 
         g_free ( tmp_str );
         g_free ( tmp_str_2 );
@@ -642,7 +640,10 @@ gboolean debug_start_log ( void )
 void debug_finish_log ( void )
 {
     if ( debug_file )
+    {
         fclose (debug_file);
+        debug_file = NULL;
+    }
 }
 
 
@@ -691,6 +692,11 @@ void debug_print_log_string ( const gchar *prefixe,
  *
  * \return
  **/
+gboolean debug_get_debug_mode (void)
+{
+	return debug_mode;
+}
+
 /* Local Variables: */
 /* c-basic-offset: 4 */
 /* End: */
