@@ -1,7 +1,7 @@
 /* ************************************************************************** */
 /*                                                                            */
 /*     Copyright (C)	2000-2008 Cédric Auger (cedric@grisbi.org)	      */
-/* 			http://www.grisbi.org				      */
+/* 			https://www.grisbi.org				      */
 /*                                                                            */
 /*  This program is free software; you can redistribute it and/or modify      */
 /*  it under the terms of the GNU General Public License as published by      */
@@ -570,7 +570,7 @@ GtkWidget *file_obfuscate_page_2 ( void )
                         "at http://grisbi.tuxfamily.org/mantis/main_page.php\n\n"
                         "The account is saved in text, you may double check with a text editor if there "
                         "is no personal information anymore in this file."
-                        "Grisbi will close immediately after saving the obfuscated file"),
+                        "Grisbi will close immediately after saving the obfuscated file."),
                         filename );
     gtk_text_buffer_insert ( buffer, &iter, text, -1 );
 
