@@ -6,7 +6,7 @@
 /*     Copyright (C)    2000-2008 Cédric Auger (cedric@grisbi.org)               */
 /*                      2003-2008 Benjamin Drieu (bdrieu@april.org)              */
 /*          2008-2017 Pierre Biava (grisbi@pierre.biava.name)                    */
-/*          http://www.grisbi.org                                                */
+/*          https://www.grisbi.org/                                              */
 /*                                                                               */
 /*     This program is free software; you can redistribute it and/or modify      */
 /*     it under the terms of the GNU General Public License as published by      */
@@ -27,7 +27,6 @@
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
-
 #include "include.h"
 #include <stdlib.h>
 #include <glib/gprintf.h>
@@ -36,27 +35,39 @@
 
 /*START_INCLUDE*/
 #include "grisbi_app.h"
-#include "grisbi_settings.h"
+#include "dialog.h"
 #include "gsb_assistant_first.h"
 #include "gsb_dirs.h"
 #include "gsb_file.h"
+#ifdef USE_CONFIG_FILE
+#include "grisbi_conf.h"
+#else
+#include "grisbi_settings.h"
+#endif
+#include "gsb_file_save.h"
 #include "gsb_locale.h"
 #include "gsb_rgba.h"
 #include "gsb_select_icon.h"
 #include "help.h"
 #include "import.h"
 #include "menu.h"
+#include "navigation.h"
 #include "structures.h"
 #include "tip.h"
 #include "traitement_variables.h"
 #include "utils_files.h"
+#include "utils_prefs.h"
 #include "utils_str.h"
 #include "erreur.h"
 /*END_INCLUDE*/
 
 /*START_STATIC*/
+static GtkCssProvider *	css_provider = NULL;    /* css provider */
+static gchar *			css_data = NULL;		/* fichier css sous forme de string */
+static gboolean			darkmode = FALSE;		/* use to set darkmode from command_line */
+
 static GrisbiWin *grisbi_app_create_window (GrisbiApp *app,
-                                             GdkScreen *screen);
+                                            GdkScreen *screen);
 /*END_STATIC*/
 
 /*START_EXTERN*/
@@ -79,6 +90,9 @@ struct _GrisbiAppPrivate
     GMenu *				item_recent_files;
     GMenu *				item_edit;
     GAction *			prefs_action;
+
+	/* config structure */
+	GrisbiAppConf		*a_conf;
 };
 
 G_DEFINE_TYPE_WITH_PRIVATE (GrisbiApp, grisbi_app, GTK_TYPE_APPLICATION)
@@ -90,10 +104,6 @@ struct AcceleratorStruct
 	const gchar *translate_string;
 };
 
-/* global variable, see structures.h */
-struct GrisbiAppConf    conf;                   /* conf structure Provisoire */
-GtkCssProvider *        css_provider = NULL;    /* css provider */
-
 static struct AcceleratorStruct accels[] = {
 		{ "app.new-window", { "<Alt>n", NULL }, N_("New_window")},
 		{ "app.prefs", { "<Primary><Shift>p", NULL }, N_("Preferences")},
@@ -140,15 +150,21 @@ static struct AcceleratorStruct accels_classic[] = {
  *
  * \return
  **/
-static void grisbi_app_struct_conf_init (void)
+static void grisbi_app_struct_conf_init (GrisbiApp *app)
 {
-    devel_debug (NULL);
+    GrisbiAppPrivate *priv;
 
-    conf.font_string = NULL;
-    conf.browser_command = NULL;
-	conf.import_directory = NULL;
-	conf.language_chosen = NULL;
-    conf.last_open_file = NULL;
+	devel_debug (NULL);
+	priv = grisbi_app_get_instance_private (GRISBI_APP (app));
+
+	priv->a_conf = g_malloc0 (sizeof (GrisbiAppConf));
+
+    (priv->a_conf)->browser_command = NULL;
+	(priv->a_conf)->current_theme = g_strdup ("null");
+    (priv->a_conf)->font_string = NULL;
+	(priv->a_conf)->import_directory = NULL;
+	(priv->a_conf)->language_chosen = NULL;
+    (priv->a_conf)->last_open_file = NULL;
 }
 
 /**
@@ -158,42 +174,47 @@ static void grisbi_app_struct_conf_init (void)
  *
  * \return
  **/
-static void grisbi_app_struct_conf_free (void)
+static void grisbi_app_struct_conf_free (GrisbiApp *app)
 {
-    devel_debug (NULL);
+    GrisbiAppPrivate *priv;
 
-	if (conf.font_string)
+	devel_debug (NULL);
+	priv = grisbi_app_get_instance_private (GRISBI_APP (app));
+
+	g_free ((priv->a_conf)->current_theme);
+	if ((priv->a_conf)->font_string)
     {
-		g_free (conf.font_string);
-		conf.font_string = NULL;
+		g_free ((priv->a_conf)->font_string);
+		(priv->a_conf)->font_string = NULL;
     }
-	if (conf.browser_command)
+	if ((priv->a_conf)->browser_command)
 	{
-        g_free (conf.browser_command);
-		conf.browser_command = NULL;
+        g_free ((priv->a_conf)->browser_command);
+		(priv->a_conf)->browser_command = NULL;
 	}
 
-    if (conf.import_directory)
+    if ((priv->a_conf)->import_directory)
     {
-        g_free (conf.import_directory);
-        conf.import_directory = NULL;
+        g_free ((priv->a_conf)->import_directory);
+        (priv->a_conf)->import_directory = NULL;
     }
 
-    if (conf.language_chosen)
+    if ((priv->a_conf)->language_chosen)
     {
-        g_free (conf.language_chosen);
-        conf.language_chosen = NULL;
+        g_free ((priv->a_conf)->language_chosen);
+        (priv->a_conf)->language_chosen = NULL;
     }
 
-	if (conf.last_open_file)
+	if ((priv->a_conf)->last_open_file)
     {
-        g_free (conf.last_open_file);
-        conf.last_open_file = NULL;
+        g_free ((priv->a_conf)->last_open_file);
+        (priv->a_conf)->last_open_file = NULL;
     }
 
+	g_free (priv->a_conf);
+
 	gsb_file_free_last_path ();
 	gsb_file_free_backup_path ();
-
 }
 
 /* ACCELERATORS*/
@@ -232,6 +253,31 @@ static void grisbi_app_setup_accelerators (GApplication *application,
 }
 
 /* MENU APP*/
+/**
+ *
+ *
+ * \param
+ * \param
+ *
+ * \return
+ **/
+static void grisbi_app_save_win_geometry_data (GrisbiApp *app,
+											   GtkWindow *win)
+{
+    GrisbiAppPrivate *priv;
+
+	priv = grisbi_app_get_instance_private (GRISBI_APP (app));
+
+	if ((priv->a_conf)->full_screen == 0 && (priv->a_conf)->maximize_screen == 0)
+	{
+		/* sauvegarde la position de la fenetre principale */
+		gtk_window_get_position (GTK_WINDOW (win), &(priv->a_conf)->x_position, &(priv->a_conf)->y_position);
+
+		/* sauvegarde de la taille de la fenêtre si nécessaire */
+		gtk_window_get_size (GTK_WINDOW (win), &(priv->a_conf)->main_width, &(priv->a_conf)->main_height);
+	}
+}
+
 /**
  *
  *
@@ -246,17 +292,21 @@ static void grisbi_app_change_fullscreen_state (GSimpleAction *action,
 												gpointer user_data)
 {
     GtkWindow *win;
+    GrisbiApp *app;
+	GrisbiAppPrivate *priv;
 
-    win = GTK_WINDOW (grisbi_app_get_active_window (user_data));
+	app = GRISBI_APP (user_data);
+	priv = grisbi_app_get_instance_private (GRISBI_APP (app));
 
+    win = GTK_WINDOW (grisbi_app_get_active_window (app));
     if (g_variant_get_boolean (state))
     {
-        conf.full_screen = TRUE;
+        (priv->a_conf)->full_screen = TRUE;
         gtk_window_fullscreen (win);
     }
     else
     {
-        conf.full_screen = FALSE;
+        (priv->a_conf)->full_screen = FALSE;
         gtk_window_unfullscreen (win);
     }
 
@@ -279,6 +329,15 @@ static void grisbi_app_change_radio_state (GSimpleAction *action,
     g_simple_action_set_state (action, state);
 }
 
+/**
+ *
+ *
+ * \param GSimpleAction     action
+ * \param GVariant          parameter
+ * \param gpointer          user_data
+ *
+ * \return
+ **/
 static void grisbi_app_new_window (GSimpleAction *action,
                                    GVariant      *parameter,
                                    gpointer       user_data)
@@ -289,6 +348,15 @@ static void grisbi_app_new_window (GSimpleAction *action,
     grisbi_app_create_window (GRISBI_APP (app), NULL);
 }
 
+/**
+ *
+ *
+ * \param
+ * \param
+ * \param
+ *
+ * \return
+ **/
 static void grisbi_app_quit (GSimpleAction *action,
                              GVariant      *parameter,
                              gpointer       user_data)
@@ -299,24 +367,21 @@ static void grisbi_app_quit (GSimpleAction *action,
 
     app = GRISBI_APP (user_data);
 
+	/* do not exit while saving. See bug #1969 */
+	if (run.menu_save)
+		return;
+
     /* Remove all windows registered in the application */
     while ((l = gtk_application_get_windows (GTK_APPLICATION (app))))
     {
         if (first_win)
         {
-            if (l->data && conf.full_screen == 0 && conf.maximize_screen == 0)
-            {
-                /* sauvegarde la position de la fenetre principale */
-                gtk_window_get_position (GTK_WINDOW (l->data), &conf.x_position, &conf.y_position);
-
-                /* sauvegarde de la taille de la fenêtre si nécessaire */
-                gtk_window_get_size (GTK_WINDOW (l->data), &conf.main_width, &conf.main_height);
-            }
+			grisbi_app_save_win_geometry_data (app, GTK_WINDOW (l->data));
             first_win = FALSE;
         }
-        if (gsb_file_close ())
+        if (gsb_file_quit ())
 		{
-			gtk_window_close (GTK_WINDOW (l->data));
+			grisbi_win_close_window (GTK_WINDOW (l->data));
 			gtk_application_remove_window (GTK_APPLICATION (app), GTK_WINDOW (l->data));
 		}
 		else
@@ -382,6 +447,7 @@ static const GActionEntry win_context_enabled_entries[] =
     { "move-to-acc", grisbi_cmd_move_to_account_menu, NULL, NULL, NULL },
 	{ "new-acc", grisbi_cmd_new_acc, NULL, NULL, NULL },
 	{ "remove-acc", grisbi_cmd_remove_acc, NULL, NULL, NULL },
+	{ "search-acc", grisbi_cmd_search_acc, NULL, NULL, NULL },
 	{ "show-form", grisbi_cmd_show_form_toggle, NULL, "false", NULL },
 	{ "show-reconciled", grisbi_cmd_show_reconciled_toggle, NULL, "false", NULL },
 	{ "show-archived", grisbi_cmd_show_archived_toggle, NULL, "false", NULL },
@@ -418,7 +484,12 @@ static void grisbi_app_init_recent_files_menu (GrisbiApp *app)
 
     priv = grisbi_app_get_instance_private (GRISBI_APP (app));
 
-	for (i = 0 ; i < conf.nb_derniers_fichiers_ouverts ; i++)
+	if (!priv->recent_array || (priv->recent_array && g_strv_length (priv->recent_array) == 0))
+	{
+		return;
+	}
+
+	for (i = 0 ; i < (priv->a_conf)->nb_derniers_fichiers_ouverts ; i++)
 	{
 		if (g_file_test (priv->recent_array[i], G_FILE_TEST_EXISTS))
 		{
@@ -437,17 +508,17 @@ static void grisbi_app_init_recent_files_menu (GrisbiApp *app)
 		{
 			gint j;
 
-			for (j = i; j < conf.nb_derniers_fichiers_ouverts -1; j++)
+			for (j = i; j < (priv->a_conf)->nb_derniers_fichiers_ouverts -1; j++)
 			{
 				priv->recent_array[j] = priv->recent_array[j+1];
 			}
 		}
 	}
-	if (index < conf.nb_derniers_fichiers_ouverts)
+	if (index < (priv->a_conf)->nb_derniers_fichiers_ouverts)
 	{
 		priv->recent_array = g_realloc (priv->recent_array, (index + 1) * sizeof (gchar*));
 		priv->recent_array[index] = NULL;
-		conf.nb_derniers_fichiers_ouverts = index;
+		(priv->a_conf)->nb_derniers_fichiers_ouverts = index;
 	}
 }
 
@@ -527,11 +598,12 @@ static void grisbi_app_set_main_menu (GrisbiApp *app,
  * \return
  **/
 static gboolean grisbi_app_window_delete_event (GrisbiWin *win,
-                        GdkEvent *event,
-                        GrisbiApp *app)
+												GdkEvent *event,
+												GrisbiApp *app)
 {
 	GList *l;
 	gboolean last_win = FALSE;
+	gboolean result = FALSE;
 
 	devel_debug (NULL);
 	l = gtk_application_get_windows (GTK_APPLICATION (app));
@@ -540,18 +612,15 @@ static gboolean grisbi_app_window_delete_event (GrisbiWin *win,
 		last_win = TRUE;
 	}
 
-    if (conf.full_screen == 0 && conf.maximize_screen == 0)
-    {
-        /* sauvegarde la position de la fenetre principale */
-        gtk_window_get_position (GTK_WINDOW (win), &conf.x_position, &conf.y_position);
+	grisbi_app_save_win_geometry_data (app, GTK_WINDOW (win));
+	if (last_win)
+		result = gsb_file_quit ();
+	else
+		result = gsb_file_close ();
 
-        /* sauvegarde de la taille de la fenêtre si nécessaire */
-        gtk_window_get_size (GTK_WINDOW (win), &conf.main_width, &conf.main_height);
-    }
-
-    if (gsb_file_close ())
+	if (result)
 	{
-		gtk_window_close (GTK_WINDOW (win));
+		grisbi_win_close_window (GTK_WINDOW (win));
 		gtk_application_remove_window (GTK_APPLICATION (app), GTK_WINDOW (win));
 
 		if (last_win)
@@ -573,28 +642,32 @@ static gboolean grisbi_app_window_delete_event (GrisbiWin *win,
 static GrisbiWin *grisbi_app_create_window (GrisbiApp *app,
 											GdkScreen *screen)
 {
+#if GTK_CHECK_VERSION (3,22,0)
+	GdkWindow *window;
+	GdkDisplay *display;
+	GdkMonitor *monitor;
+	GdkRectangle rectangle;
+#endif
     GrisbiWin *win;
     GrisbiAppPrivate *priv;
-    gchar *string;
 
     priv = grisbi_app_get_instance_private (GRISBI_APP (app));
 
-	win = g_object_new (GRISBI_TYPE_WIN, "application", app, NULL);
+	win = g_object_new (GRISBI_WIN_TYPE, "application", app, NULL);
 
     g_signal_connect (win,
-                        "delete_event",
-                        G_CALLBACK (grisbi_app_window_delete_event),
-                        app);
+					  "delete_event",
+					  G_CALLBACK (grisbi_app_window_delete_event),
+					  app);
+
+	/* set signal for detecting new theme */
+    g_signal_connect (win,
+					  "style-updated",
+					  G_CALLBACK (grisbi_app_window_style_updated),
+					  GINT_TO_POINTER (FALSE));
 
 	gtk_application_add_window (GTK_APPLICATION (app), GTK_WINDOW (win));
 
-    /* create the icon of grisbi (set in the panel of gnome or other) */
-    string = g_build_filename (gsb_dirs_get_pixmaps_dir (), "grisbi-logo.png", NULL);
-    if (g_file_test (string, G_FILE_TEST_EXISTS))
-        gtk_window_set_default_icon_from_file (string, NULL);
-
-    g_free (string);
-
 	/* Adding pixmaps_dir in the icon theme */
 	gsb_select_icon_set_gtk_icon_theme_path ();
 
@@ -632,6 +705,28 @@ static GrisbiWin *grisbi_app_create_window (GrisbiApp *app,
 	if (screen != NULL)
         gtk_window_set_screen (GTK_WINDOW (win), screen);
 
+	/* on teste s'il faut changer de résolution */
+#if GTK_CHECK_VERSION (3,22,0)
+	if (!(priv->a_conf)->low_definition_screen)
+	{
+		window = gtk_widget_get_window (GTK_WIDGET (win));
+		display = gdk_window_get_display (GDK_WINDOW (window));
+		monitor = gdk_display_get_monitor_at_point (display, 0, 0);
+		gdk_monitor_get_geometry (monitor, &rectangle);
+
+		if (rectangle.width < LOW_DEF_WIDTH_HIGH || rectangle.height < LOW_DEF_HEIGHT_HIGH)
+		{
+			(priv->a_conf)->low_definition_screen = TRUE;
+			(priv->a_conf)->main_height = WIN_MIN_HEIGHT;
+			(priv->a_conf)->main_width = WIN_MIN_WIDTH;
+			(priv->a_conf)->panel_width = PANEL_MIN_WIDTH;
+
+			gtk_window_resize (GTK_WINDOW (grisbi_app_get_active_window (NULL)),
+							   (priv->a_conf)->main_width,
+							   (priv->a_conf)->main_height);
+		}
+	}
+#endif
     return win;
 }
 
@@ -657,6 +752,13 @@ static const GOptionEntry options[] =
 		N_("DEBUG")
     },
 
+	/* darkmode */
+	{
+		"darkmode", 'm', 0, G_OPTION_ARG_NONE, NULL,
+		N_("If present, force the use of a dark theme"),
+		NULL
+	},
+
 	/* New instance */
 /*	{
 		"standalone", 's', 0, G_OPTION_ARG_NONE, NULL,
@@ -664,6 +766,14 @@ static const GOptionEntry options[] =
 		NULL
 	},
 */
+#ifdef __APPLE__
+    /* communicate with the macOS launcher */
+    {
+        "launcher", '\0', 0, G_OPTION_ARG_STRING, NULL,
+        "macOS launcher synchronisation pipe",
+        "FILENAME"
+    },
+#endif
 	/* collects file arguments */
 	{
 		G_OPTION_REMAINING, '\0', 0, G_OPTION_ARG_FILENAME_ARRAY, NULL, NULL,
@@ -716,9 +826,12 @@ static gboolean grisbi_app_cmdline (GApplication *application,
 									GApplicationCommandLine *cmdline)
 {
 	GrisbiAppPrivate *priv;
-	GVariantDict *options;
+	GVariantDict *v_options;
 	gchar *tmp_str = NULL;
 	const gchar **remaining_args;
+#ifdef __APPLE__
+    gchar *launcher_str = NULL;
+#endif
 
 	priv = grisbi_app_get_instance_private (GRISBI_APP (application));
 
@@ -726,14 +839,17 @@ static gboolean grisbi_app_cmdline (GApplication *application,
 	priv->debug_level = -1;
 
 	/* traitement des autres options */
-	options = g_application_command_line_get_options_dict (cmdline);
+	v_options = g_application_command_line_get_options_dict (cmdline);
 
-	g_variant_dict_lookup (options, "new-window", "b", &priv->new_window);
-	g_variant_dict_lookup (options, "debug", "s", &tmp_str);
-	g_variant_dict_lookup (options, "d", "s", &tmp_str);
+	g_variant_dict_lookup (v_options, "new-window", "b", &priv->new_window);
+	g_variant_dict_lookup (v_options, "debug", "s", &tmp_str);
+	g_variant_dict_lookup (v_options, "d", "s", &tmp_str);
+#ifdef __APPLE__
+    g_variant_dict_lookup (v_options, "launcher", "s", &launcher_str);
+#endif
 
     /* Parse filenames */
-	if (g_variant_dict_lookup (options, G_OPTION_REMAINING, "^a&ay", &remaining_args))
+	if (g_variant_dict_lookup (v_options, G_OPTION_REMAINING, "^a&ay", &remaining_args))
 	{
 		gint i;
 
@@ -754,29 +870,50 @@ static gboolean grisbi_app_cmdline (GApplication *application,
 	/* modification du niveau de débug si besoin */
 	if (tmp_str && strlen (tmp_str) > 0)
 	{
-		gchar *endptr;
 		gint64 number;
 
 		errno = 0;
-		number = g_ascii_strtoll (tmp_str, &endptr, 10);
-		if (endptr == NULL)
+		number = g_ascii_strtoll (tmp_str, NULL, 10);
+		if (number > 0)
 			priv->debug_level = (gint)number;
 		else if (errno == 0 && number == 0)
 			priv->debug_level = (gint)number;
 	}
 
-	if (IS_DEVELOPMENT_VERSION == 1)
-    {
-		if (priv->debug_level >= 0 && priv->debug_level < 5)
+#ifdef DEBUG
+		/* On peut forcer le niveau 0 (pas de debug) */
+		if (priv->debug_level >= 0 && priv->debug_level < MAX_DEBUG_LEVEL)
 			debug_set_cmd_line_debug_level (priv->debug_level);
-    }
-	else if (priv->debug_level > 0)
-	{
-		debug_initialize_debugging (priv->debug_level);
-        grisbi_app_print_environment_var ();
-	}
+#else
+		if (priv->debug_level > 0)
+		{
+			debug_initialize_debugging (priv->debug_level);
+			grisbi_app_print_environment_var ();
+		}
+#endif
 
-	if (priv->new_window)
+#ifdef __APPLE__
+    if (launcher_str && strlen(launcher_str) > 0)
+    {
+        FILE *fd;
+        fd = fopen(launcher_str, "w");
+        if (fd == NULL)
+        {
+            char *error;
+            asprintf(&error, "%s: %s", launcher_str, strerror(errno));
+            important_debug(error);
+            free(error);
+        }
+        else
+        {
+            char ok[] = "ok\n";
+            fwrite(ok, sizeof(ok), 1, fd);
+            fclose(fd);
+        }
+    }
+#endif
+
+    if (priv->new_window)
 		grisbi_app_create_window (GRISBI_APP (application), NULL);
 
 	g_application_activate (application);
@@ -792,9 +929,9 @@ static gboolean grisbi_app_cmdline (GApplication *application,
  * \return
  **/
 static gint grisbi_app_handle_local_options (GApplication *app,
-											 GVariantDict *options)
+											 GVariantDict *v_options)
 {
-    if (g_variant_dict_contains (options, "version"))
+    if (g_variant_dict_contains (v_options, "version"))
     {
         g_print ("%s - Version %s\n", g_get_application_name (), VERSION);
         g_print("\n\n");
@@ -802,6 +939,12 @@ static gint grisbi_app_handle_local_options (GApplication *app,
         return 0;
     }
 
+	if (g_variant_dict_contains (v_options, "darkmode"))
+	{
+		darkmode = TRUE;
+		g_print ("darkmode is set\n");
+	}
+
     return -1;
 }
 
@@ -871,12 +1014,12 @@ static gboolean grisbi_app_load_file_if_necessary (GrisbiApp *app)
     else
     {
         /* open the last file if needed, filename was set while loading the configuration */
-        if (conf.dernier_fichier_auto && conf.last_open_file)
+        if ((priv->a_conf)->dernier_fichier_auto && (priv->a_conf)->last_open_file)
         {
-            if (!gsb_file_open_file (conf.last_open_file))
+            if (!gsb_file_open_file ((priv->a_conf)->last_open_file))
             {
-                g_free (conf.last_open_file);
-                conf.last_open_file = NULL;
+                g_free ((priv->a_conf)->last_open_file);
+                (priv->a_conf)->last_open_file = NULL;
 
                 return FALSE;
             }
@@ -903,52 +1046,91 @@ static void grisbi_app_startup (GApplication *application)
 {
 	GrisbiApp *app = GRISBI_APP (application);
     GFile *file = NULL;
-    gchar *tmp_dir;
     GtkSettings* settings;
+	gchar *css_filename;
+	gchar *theme_name = NULL;
 	gboolean has_app_menu = FALSE;
+    GrisbiAppPrivate *priv;
 
 	/* Chain up parent's startup */
     G_APPLICATION_CLASS (grisbi_app_parent_class)->startup (application);
 
+	priv = grisbi_app_get_instance_private (GRISBI_APP (application));
+
+	/* on commence par détourner le signal SIGSEGV */
+    grisbi_app_trappe_signaux ();
+
+    /* initialisation de la variable conf */
+    grisbi_app_struct_conf_init (app);
+
+    /* initialisation des variables de configuration globales */
+#ifdef USE_CONFIG_FILE
+	grisbi_conf_load_app_config ();
+#else
+	grisbi_settings_load_app_config ();
+#endif
+
+	/* force dark type */
+	if (darkmode)
+	{
+		(priv->a_conf)->force_type_theme = 2;
+	}
+
 	settings = gtk_settings_get_default ();
     if (settings)
     {
         g_object_get (G_OBJECT (settings),
                       "gtk-shell-shows-app-menu", &has_app_menu,
-                      NULL);
+					  "gtk-theme-name", &theme_name,
+					  NULL);
+
+		if (g_strcmp0 ((priv->a_conf)->current_theme, theme_name) == 0)
+		{
+
+			if ((priv->a_conf)->force_type_theme)
+				(priv->a_conf)->use_type_theme = (priv->a_conf)->force_type_theme;
+			else	/* mode automatique */
+				(priv->a_conf)->use_type_theme = gsb_rgba_get_type_theme (theme_name);
+
+			g_free (theme_name);
+		}
+		else
+		{
+			g_free ((priv->a_conf)->current_theme);
+			(priv->a_conf)->current_theme = theme_name;
+			(priv->a_conf)->use_type_theme = gsb_rgba_get_type_theme (theme_name);
+			(priv->a_conf)->force_type_theme = 0;
+		}
     }
 
-    /* on commence par détourner le signal SIGSEGV */
-    grisbi_app_trappe_signaux ();
-
-    /* initialisation des variables de configuration globales */
-    grisbi_settings_get ();
-
 	/* set language and init locale parameters */
-	gsb_locale_init_language (conf.language_chosen);
+	gsb_locale_init_language ((priv->a_conf)->language_chosen);
 	gsb_locale_init_lconv_struct ();
 
 	/* Print variables if necessary */
-	if (IS_DEVELOPMENT_VERSION == 1)
-    {
-        grisbi_app_print_environment_var ();
-    }
+#ifdef DEBUG
+	grisbi_app_print_environment_var ();
+#endif
 
-	/* MAJ de has_app_menu */
-    if (conf.force_classic_menu)
-        has_app_menu = FALSE;
+#ifdef OS_OSX
+	has_app_menu = TRUE;
+#else
+	has_app_menu = FALSE;
+#endif /* OS_OSX */
 
-    /* load the CSS properties */
+	/* load the CSS properties */
     css_provider = gtk_css_provider_new ();
-    tmp_dir = g_strconcat (gsb_dirs_get_ui_dir (), "/grisbi.css", NULL);
-    file = g_file_new_for_path (tmp_dir);
-    if (!gtk_css_provider_load_from_file (css_provider, file, NULL))
-        warning_debug (tmp_dir);
-    g_free (tmp_dir);
+	css_filename = gsb_css_get_filename ();
+    file = g_file_new_for_path (css_filename);
+    gtk_css_provider_load_from_file (css_provider, file, NULL);
+    g_free (css_filename);
 	g_object_unref (file);
 
+	/* récupération des données du fichier grisbi.css*/
+	css_data = gtk_css_provider_to_string (css_provider);
+
     /* initialise les couleurs */
-    gsb_rgba_initialise_couleurs_par_defaut ();
+    gsb_rgba_initialise_couleurs (css_data);
 
 	/* enregistre les formats d'importation */
     gsb_import_register_import_formats ();
@@ -971,24 +1153,33 @@ static void grisbi_app_activate (GApplication *application)
 {
 
 	GrisbiWin *win;
+    //~ gchar *string;
     gboolean load_file = FALSE;
+	GrisbiAppPrivate *priv;
 
 	devel_debug (NULL);
+	priv = grisbi_app_get_instance_private (GRISBI_APP (application));
 
 	/* création de la fenêtre pincipale */
     win = grisbi_app_create_window (GRISBI_APP (application), NULL);
 
-    /* set the CSS properties */
+    /* create the icon of grisbi (set in the panel of gnome or other) */
+    //~ string = g_build_filename (gsb_dirs_get_pixmaps_dir (), "grisbi.svg", NULL);
+    //~ if (g_file_test (string, G_FILE_TEST_EXISTS))
+        //~ gtk_window_set_default_icon_from_file (string, NULL);
+    //~ g_free (string);
+
+	/* set the CSS properties */
     if (css_provider)
         gtk_style_context_add_provider_for_screen (gdk_screen_get_default (),
                                                    GTK_STYLE_PROVIDER (css_provider),
                                                    GTK_STYLE_PROVIDER_PRIORITY_USER);
 
     /* lance un assistant si première utilisation */
-    if (conf.first_use)
+    if ((priv->a_conf)->first_use)
     {
         gsb_assistant_first_run ();
-        conf.first_use = FALSE;
+        (priv->a_conf)->first_use = FALSE;
     }
     else
     {
@@ -1002,6 +1193,12 @@ static void grisbi_app_activate (GApplication *application)
 			grisbi_win_status_bar_message (_("Start an action!"));
 		}
     }
+
+    /* Si sauvegarde automatique on la lance ici */
+	if ((priv->a_conf)->make_backup_every_minutes
+        &&
+        (priv->a_conf)->make_backup_nb_minutes)
+        gsb_file_automatic_backup_start (NULL, NULL);
 }
 
 /**
@@ -1037,6 +1234,26 @@ static void grisbi_app_open (GApplication *application,
 /******************************************************************************/
 /* Fonctions propres à l'initialisation de l'application                      */
 /******************************************************************************/
+/**
+ * grisbi_app_init
+ *
+ * \param GApplication *app
+ *
+ * \return
+ **/
+static void grisbi_app_init (GrisbiApp *app)
+{
+    /* initialize debugging */
+#ifdef DEBUG
+	debug_initialize_debugging (5);
+#endif
+
+    g_set_application_name ("Grisbi");
+
+	/* add options for app */
+    g_application_add_main_option_entries (G_APPLICATION (app), options);
+}
+
 /**
  * grisbi_app_dispose
  *
@@ -1053,8 +1270,10 @@ static void grisbi_app_dispose (GObject *object)
 	priv = grisbi_app_get_instance_private (GRISBI_APP (object));
 
     /* liberation de la mémoire utilisée par les objets de priv*/
-    g_clear_object (&priv->appmenu);
-    g_clear_object (&priv->menubar);
+	if (priv->appmenu)
+		g_clear_object (&priv->appmenu);
+	else
+		g_clear_object (&priv->menubar);
 
 	if (priv->recent_array && g_strv_length (priv->recent_array) > 0)
 		g_strfreev (priv->recent_array);
@@ -1071,14 +1290,16 @@ static void grisbi_app_dispose (GObject *object)
  **/
 static void grisbi_app_shutdown (GApplication *application)
 {
+    GrisbiAppPrivate *priv;
+
+    priv = grisbi_app_get_instance_private (GRISBI_APP (application));
 	devel_debug (NULL);
 
-    /* clean finish of the debug file */
-    if (conf.debug_mode)
-	{
-		conf.debug_mode = FALSE;
-        debug_finish_log ();
-	}
+	/* on sauvegarde éventuellement le fichier CSS local */
+	gsb_file_save_css_local_file (css_data);
+
+	/* on libère la mémoire utilisée par css_data */
+	g_free (css_data);
 
     /* on libère la mémoire utilisée par etat */
     free_variables ();
@@ -1086,40 +1307,33 @@ static void grisbi_app_shutdown (GApplication *application)
 	/* libération de mémoire utilisée par locale*/
     gsb_locale_shutdown ();
 
+	/* on remet la détection du theme en auto si darkmode = TRUE */
+	if (darkmode)
+		(priv->a_conf)->force_type_theme = 0;
+
+    /* Sauvegarde de la configuration générale */
+#ifdef USE_CONFIG_FILE
+	grisbi_conf_save_app_config ();
+#else
+	grisbi_settings_save_app_config ();
+	grisbi_settings_save_in_config_file ();
+#endif
+
 	/* libération de mémoire utilisée par gsb_dirs*/
     gsb_dirs_shutdown ();
 
-    /* Sauvegarde de la configuration générale */
-    grisbi_settings_save_app_config ();
-
 	/* on libère la mémoire utilisée par conf */
-    grisbi_app_struct_conf_free ();
+    grisbi_app_struct_conf_free (GRISBI_APP (application));
+
+    /* clean finish of the debug file */
+    if (debug_get_debug_mode ())
+	{
+        debug_finish_log ();
+	}
 
     G_APPLICATION_CLASS (grisbi_app_parent_class)->shutdown (application);
 }
 
-/**
- * grisbi_app_init
- *
- * \param GApplication *app
- *
- * \return
- **/
-static void grisbi_app_init (GrisbiApp *app)
-{
-    /* initialize debugging */
-    if (IS_DEVELOPMENT_VERSION == 1)
-        debug_initialize_debugging (5);
-
-    g_set_application_name ("Grisbi");
-
-	/* add options for app */
-    g_application_add_main_option_entries (G_APPLICATION (app), options);
-
-    /* initialisation de la variable conf */
-    grisbi_app_struct_conf_init ();
-}
-
 /**
  * grisbi_app_class_init
  *
@@ -1145,6 +1359,77 @@ static void grisbi_app_class_init (GrisbiAppClass *klass)
 /******************************************************************************/
 /* Public functions                                                           */
 /******************************************************************************/
+void grisbi_app_window_style_updated (GtkWidget *win,
+									  gpointer force)
+{
+    GtkSettings* settings;
+	gint forced = 0;
+
+	g_signal_handlers_block_by_func (G_OBJECT (win),
+									 G_CALLBACK (grisbi_app_window_style_updated),
+									 force);
+
+	forced = GPOINTER_TO_INT (force);
+	settings = gtk_settings_get_default ();
+    if (settings)
+    {
+		GFile *file = NULL;
+		gchar *tmp_theme_name;
+		const gchar *css_filename;
+		GrisbiAppConf *a_conf;
+
+		a_conf = grisbi_app_get_a_conf ();
+
+		g_object_get (G_OBJECT (settings),
+					  "gtk-theme-name", &tmp_theme_name,
+					  NULL);
+
+		if (g_strcmp0 (tmp_theme_name, a_conf->current_theme) || forced)
+		{
+			g_free (a_conf->current_theme);
+			a_conf->current_theme = tmp_theme_name;
+
+			/* set force_type_theme en automatique sauf si commande forcée */
+			if (!forced)
+			{
+				a_conf->use_type_theme = gsb_rgba_get_type_theme (tmp_theme_name);
+				a_conf->force_type_theme = 0;
+			}
+			else
+			{
+				a_conf->use_type_theme = a_conf->force_type_theme;
+			}
+
+			/* on sauvegarde éventuellement les données locales */
+			gsb_file_save_css_local_file (css_data);
+
+			/* on charge les nouvelles données */
+			css_filename = gsb_css_get_filename ();
+			file = g_file_new_for_path (css_filename);
+			gtk_css_provider_load_from_file (css_provider, file, NULL);
+
+			/* on met à jour css_data et les couleurs de base */
+			g_free (css_data);
+			css_data = gtk_css_provider_to_string (css_provider);
+			gsb_rgba_initialise_couleurs (css_data);
+
+			/* MAJ Home page */
+			if (grisbi_win_file_is_loading ())
+				gsb_gui_navigation_update_home_page_from_theme ();
+
+			/* on ferme les preferences si necessaire */
+			utils_prefs_close_prefs_from_theme ();
+		}
+		else
+		{
+			g_free (tmp_theme_name);
+		}
+	}
+	g_signal_handlers_unblock_by_func (G_OBJECT (win),
+									   G_CALLBACK (grisbi_app_window_style_updated),
+									   force);
+}
+
 /**
  * Affiche les raccourcis claviers dans un text view
  *
@@ -1201,6 +1486,17 @@ void grisbi_app_display_gui_dump_accels (GtkApplication *application,
 	gtk_text_buffer_insert (buffer, &iter, _("(Un)Reconcile a transaction\t\t-> <Primary>r\n"), -1);
 }
 
+gpointer grisbi_app_get_a_conf (void)
+{
+	GApplication *app;
+	GrisbiAppPrivate *priv;
+
+	app = g_application_get_default ();
+	priv = grisbi_app_get_instance_private (GRISBI_APP (app));
+
+	return priv->a_conf;
+}
+
 /**
  * get active window.
  *
@@ -1256,6 +1552,24 @@ GMenu *grisbi_app_get_menu_edit (void)
 	return priv->item_edit;
 }
 
+/**
+ *
+ *
+ * \param
+ *
+ * \return
+ **/
+gboolean grisbi_app_get_low_definition_screen (void)
+{
+	GApplication *app;
+	GrisbiAppPrivate *priv;
+
+	app = g_application_get_default ();
+	priv = grisbi_app_get_instance_private (GRISBI_APP (app));
+
+	return (priv->a_conf)->low_definition_screen;
+}
+
 /**
  *
  *
@@ -1374,10 +1688,10 @@ void grisbi_app_update_recent_files_menu (void)
 	/* on vide le sous menu recent_files */
 	g_menu_remove_all (priv->item_recent_files);
 
-	if (conf.nb_derniers_fichiers_ouverts > conf.nb_max_derniers_fichiers_ouverts)
-		conf.nb_derniers_fichiers_ouverts = conf.nb_max_derniers_fichiers_ouverts;
+	if ((priv->a_conf)->nb_derniers_fichiers_ouverts > (priv->a_conf)->nb_max_derniers_fichiers_ouverts)
+		(priv->a_conf)->nb_derniers_fichiers_ouverts = (priv->a_conf)->nb_max_derniers_fichiers_ouverts;
 
-	for (i = 0 ; i < conf.nb_derniers_fichiers_ouverts ; i++)
+	for (i = 0 ; i < (priv->a_conf)->nb_derniers_fichiers_ouverts ; i++)
 	{
 		gchar *menu_name;
 
@@ -1393,6 +1707,43 @@ void grisbi_app_update_recent_files_menu (void)
 	}
 }
 
+/**
+ *
+ *
+ * \param
+ *
+ * \return
+ **/
+GtkCssProvider *grisbi_app_get_css_provider (void)
+{
+	return css_provider;
+}
+
+/**
+ *
+ *
+ * \param
+ *
+ * \return
+ **/
+const gchar *grisbi_app_get_css_data (void)
+{
+	return css_data;
+}
+
+/**
+ *
+ *
+ * \param
+ *
+ * \return
+ **/
+void grisbi_app_set_css_data (const gchar *new_css_data)
+{
+	g_free (css_data);
+	css_data = g_strdup (new_css_data);
+}
+
 /**
  *
  *
