@@ -2,7 +2,7 @@
 /*                                                                            */
 /*     Copyright (C)    2000-2008 Cédric Auger (cedric@grisbi.org)            */
 /*          2003-2009 Benjamin Drieu (bdrieu@april.org)                       */
-/*          http://www.grisbi.org                                             */
+/*          https://www.grisbi.org/                                            */
 /*                                                                            */
 /*  This program is free software; you can redistribute it and/or modify      */
 /*  it under the terms of the GNU General Public License as published by      */
@@ -90,7 +90,7 @@ GtkResponseType gsb_assistant_first_run ( void )
 				      "You can change any option later in the configuration window, "
 				      "where many other options are available.\n\n"
 				      "Thanks for using Grisbi, enjoy!" ),
-				    "grisbi-32.png",
+				    "grisbi.svg",
 				    NULL );
     gsb_assistant_add_page ( assistant,
 			     gsb_assistant_first_page_2 (assistant),
@@ -154,8 +154,8 @@ GtkResponseType gsb_assistant_first_come_to_0_6 ( void )
 				      "in next screen.\n\n"
 				      "If you want to make backups of your Grisbi file in case you want to revert "
 				      "to old version of Grisbi, we would advise you to do that right now.\n\n"
-				      "You can find out other improvements on http://www.grisbi.org/."),
-				    "grisbi-32.png",
+				      "You can find out other improvements on https://www.grisbi.org/."),
+				    "grisbi.svg",
 				    NULL );
     gsb_assistant_add_page ( assistant,
 			     gsb_assistant_first_page_2 (assistant),
@@ -198,11 +198,13 @@ static GtkWidget *gsb_assistant_first_page_2 ( GtkWidget *assistant )
     gchar *text;
     GtkWidget *hbox;
     GtkWidget *dialog;
+	GrisbiAppConf *a_conf;
 	GrisbiWinEtat *w_etat;
 	GrisbiWinRun *w_run;
 
-	w_etat = grisbi_win_get_w_etat ();
-	w_run = grisbi_win_get_w_run ();
+	a_conf = (GrisbiAppConf *) grisbi_app_get_a_conf ();
+	w_etat = (GrisbiWinEtat *) grisbi_win_get_w_etat ();
+	w_run = (GrisbiWinRun *) grisbi_win_get_w_run ();
     page = gtk_box_new ( GTK_ORIENTATION_HORIZONTAL, MARGIN_BOX);
     gtk_container_set_border_width ( GTK_CONTAINER (page), BOX_BORDER_WIDTH );
 
@@ -224,10 +226,10 @@ static GtkWidget *gsb_assistant_first_page_2 ( GtkWidget *assistant )
     gtk_size_group_add_widget ( size_group, label );
     utils_labels_set_alignment ( GTK_LABEL ( label ), 0.0, 0.5 );
     gtk_grid_attach (GTK_GRID (table), label, 0, 1, 1, 1);
-    entry = gsb_automem_entry_new ( &conf.browser_command, NULL, NULL );
+    entry = gsb_automem_entry_new ( &a_conf->browser_command, NULL, NULL );
     gtk_grid_attach (GTK_GRID (table), entry, 1, 1, 1, 1);
 
-    text = make_blue ( _("You may use %s to expand the URL - I.e: 'firefox %s' ") );
+    text = dialogue_make_blue ( _("You may use %s to expand the URL - I.e: 'firefox %s' ") );
     label = gtk_label_new ( text );
     gtk_label_set_use_markup ( GTK_LABEL(label), TRUE );
     utils_labels_set_alignment ( GTK_LABEL ( label ), 0.0, 0.5 );
@@ -240,12 +242,12 @@ static GtkWidget *gsb_assistant_first_page_2 ( GtkWidget *assistant )
 
     /* Automatically load last file on startup? */
     button = gsb_automem_checkbutton_new (_("Automatically load last file on startup"),
-					  &conf.dernier_fichier_auto, NULL, NULL );
+					  &a_conf->dernier_fichier_auto, NULL, NULL );
     gtk_box_pack_start ( GTK_BOX ( paddingbox ), button, FALSE, FALSE, 0 );
 
     /* automatically save file at closing */
     button = gsb_automem_checkbutton_new (_("Automatically save on exit"),
-					  &conf.sauvegarde_auto, NULL, NULL);
+					  &a_conf->sauvegarde_auto, NULL, NULL);
     gtk_box_pack_start ( GTK_BOX ( paddingbox ), button,
 			 FALSE, FALSE, 0 );
 
@@ -254,7 +256,7 @@ static GtkWidget *gsb_assistant_first_page_2 ( GtkWidget *assistant )
     {
         button = gsb_automem_checkbutton_new (_("Encrypt Grisbi file"),
 											  &w_etat->crypt_file,
-											  G_CALLBACK (gsb_gui_encryption_toggled),
+											  G_CALLBACK (utils_prefs_encryption_toggled),
 											  NULL);
         gtk_box_pack_start ( GTK_BOX ( paddingbox ), button,
                              FALSE, FALSE, 0 );
@@ -268,7 +270,7 @@ static GtkWidget *gsb_assistant_first_page_2 ( GtkWidget *assistant )
     }
 #endif
     button = gsb_automem_checkbutton_new (_("Make a backup copy before saving files"),
-					  &conf.make_backup, NULL, NULL);
+					  &a_conf->sauvegarde_fermeture, NULL, NULL);
     gtk_box_pack_start ( GTK_BOX ( paddingbox ), button, FALSE, FALSE, 0 );
 
     /* Automatic backup every x minutes */
@@ -276,11 +278,11 @@ static GtkWidget *gsb_assistant_first_page_2 ( GtkWidget *assistant )
     gtk_box_pack_start ( GTK_BOX ( paddingbox ), hbox, FALSE, FALSE, 0);
 
     button = gsb_automem_checkbutton_new (_("Make a backup copy every "),
-					  &conf.make_backup_every_minutes,
+					  &a_conf->make_backup_every_minutes,
 					  G_CALLBACK (gsb_file_automatic_backup_start), NULL);
     gtk_box_pack_start ( GTK_BOX (hbox), button, FALSE, FALSE, 0 );
 
-    button = gsb_automem_spin_button_new ( &conf.make_backup_nb_minutes,
+    button = gsb_automem_spin_button_new ( &a_conf->make_backup_nb_minutes,
 					   G_CALLBACK (gsb_file_automatic_backup_change_time), NULL );
 
     gtk_box_pack_start ( GTK_BOX (hbox), button, FALSE, FALSE, 0 );
@@ -304,7 +306,7 @@ static GtkWidget *gsb_assistant_first_page_2 ( GtkWidget *assistant )
     g_signal_connect (G_OBJECT (button),
                       "selection-changed",
                       G_CALLBACK (utils_prefs_page_dir_chosen),
-                      "backup_path");
+                      (gpointer) "backup_path");
     gtk_box_pack_start ( GTK_BOX ( hbox ), button, FALSE, FALSE, 0);
 
     gtk_widget_show_all (page);
