@@ -3,7 +3,7 @@
 /*     Copyright (C)    2000-2008 Cédric Auger (cedric@grisbi.org)            */
 /*          2003-2008 Benjamin Drieu (bdrieu@april.org)	                      */
 /*          2008-2017 Pierre Biava (grisbi@pierre.biava.name)                 */
-/*          http://www.grisbi.org                                             */
+/*          https://www.grisbi.org/                                            */
 /*                                                                            */
 /*  This program is free software; you can redistribute it and/or modify      */
 /*  it under the terms of the GNU General Public License as published by      */
@@ -40,11 +40,9 @@
 /*START_INCLUDE*/
 #include "gsb_file.h"
 #include "accueil.h"
-#include "affichage_liste.h"
 #include "dialog.h"
 #include "etats_gtktable.h"
 #include "grisbi_app.h"
-#include "grisbi_settings.h"
 #include "gsb_account_property.h"
 #include "gsb_assistant_account.h"
 #include "gsb_data_account.h"
@@ -62,6 +60,7 @@
 #include "traitement_variables.h"
 #include "transaction_list.h"
 #include "utils.h"
+#include "utils_dates.h"
 #include "utils_files.h"
 #include "utils_str.h"
 #include "erreur.h"
@@ -82,6 +81,175 @@ gint id_timeout = 0;
 /******************************************************************************/
 /* Private Methods                                                            */
 /******************************************************************************/
+/**
+ *
+ *
+ * \param
+ *
+ * \return
+ **/
+static void gsb_file_remove_old_backup (const gchar *filename,
+										gint remove_backup_months)
+{
+	GFileEnumerator *direnum;
+	GFile *dir;
+	GDate *first_old_date;
+    GSList *filenames = NULL;
+    GSList *iterator;
+	gchar *basename;
+	gchar *name = NULL;
+
+	devel_debug (filename);
+
+	/* get name of file without extension */
+	basename = g_path_get_basename (filename);
+	if (g_str_has_suffix (basename, ".gsb"))
+	{
+		name = g_strndup (basename, strlen (basename) - 4);
+		g_free (basename);
+	}
+	else
+	{
+		g_free (basename);
+
+		return;
+	}
+
+	if (!name)
+		return;
+
+	/* get today date and first_old_date */
+	first_old_date = gdate_today ();
+	g_date_subtract_months (first_old_date, remove_backup_months);
+
+	/* balayage du répertoire de sauvegarde */
+	dir =  g_file_new_for_path (backup_path);
+	direnum = g_file_enumerate_children (dir,
+										"standard::*",
+										G_FILE_QUERY_INFO_NONE,
+										NULL,
+										NULL);
+	if (!direnum)
+		return;
+
+	while (TRUE)
+	{
+		GFileInfo *info = NULL;
+		const gchar *old_filename;
+
+		if (!g_file_enumerator_iterate (direnum, &info, NULL, NULL, NULL))
+			break;
+
+		if (!info)
+		  break;
+
+		old_filename = g_file_info_get_name (info);
+		if (old_filename && strlen (old_filename))
+		{
+			if (g_str_has_suffix (old_filename, ".gsb"))
+			{
+				if (name && g_str_has_prefix (old_filename, name))
+				{
+					GDate *date;
+					gchar *str_date;
+					gchar *tmp_year;
+					gchar *tmp_month;
+					gchar *tmp_day;
+					size_t long_name;
+
+					long_name = strlen (name);
+					str_date = g_strndup (old_filename+long_name+1, 8);
+					tmp_year = g_strndup (str_date, 4);
+					if (g_date_valid_year (utils_str_atoi (tmp_year)) == FALSE)
+					{
+						g_free (str_date);
+						g_free (tmp_year);
+
+						continue;
+					}
+					tmp_month = g_strndup (str_date+4, 2);
+					if (g_date_valid_month (utils_str_atoi (tmp_month)) == FALSE)
+					{
+						g_free (str_date);
+						g_free (tmp_year);
+						g_free (tmp_month);
+
+						continue;
+					}
+					tmp_day = g_strndup (str_date+6, 2);
+					if (g_date_valid_day (utils_str_atoi (tmp_day)) == FALSE)
+					{
+						g_free (str_date);
+						g_free (tmp_year);
+						g_free (tmp_month);
+						g_free (tmp_day);
+
+						continue;
+					}
+					date = g_date_new_dmy (utils_str_atoi (tmp_day),
+										   utils_str_atoi (tmp_month),
+										   utils_str_atoi (tmp_year));
+					if (g_date_valid (date) == FALSE)
+					{
+						g_date_free (date);
+						g_free (str_date);
+						g_free (tmp_year);
+						g_free (tmp_month);
+						g_free (tmp_day);
+
+						continue;
+					}
+					if (g_date_compare (date, first_old_date) < 0)
+					{
+						gchar *tmp_filename;
+
+						tmp_filename = g_build_filename (backup_path, G_DIR_SEPARATOR_S, old_filename, NULL);
+						filenames = g_slist_append (filenames, tmp_filename);
+					}
+					g_date_free (date);
+					g_free (str_date);
+					g_free (tmp_year);
+					g_free (tmp_month);
+					g_free (tmp_day);
+				}
+			}
+		}
+		else
+		{
+			continue;
+		}
+	}
+	g_object_unref (direnum);
+	g_object_unref (dir);
+	g_free (name);
+
+	if (!filenames)
+	{
+		g_date_free (first_old_date);
+
+		return;
+	}
+
+	dialogue_message ("remove-backup-files", g_slist_length (filenames), gsb_format_gdate (first_old_date));
+
+	g_date_free (first_old_date);
+
+	iterator = filenames;
+	while (iterator)
+    {
+		GFile *file;
+		gchar *path;
+
+		path = iterator->data;
+
+		file = g_file_new_for_path (path);
+		g_file_delete (file, FALSE, NULL);
+		iterator = iterator->next;
+	}
+
+	g_slist_free_full (filenames, g_free);
+}
+
 /**
  * teste la validité d'un fichier
  *
@@ -224,7 +392,7 @@ static gint gsb_file_dialog_save (const gchar *filename,
 			gint difference;
 
 			now = time (NULL);
-			difference = (gint) difftime (now, run.file_modification);
+			difference = (int) (difftime (now, run.file_modification));
 			hint = g_strdup_printf (_("Save changes to document '%s' before closing?"),
 									(filename ? g_path_get_basename(filename) : _("unnamed")));
 
@@ -287,7 +455,8 @@ static gint gsb_file_dialog_save (const gchar *filename,
  *
  * \return TRUE ok, FALSE problem
  **/
-static gboolean gsb_file_save_backup (void)
+static gboolean gsb_file_save_backup (gboolean make_bakup_single_file,
+									  gboolean compress_backup)
 {
     gboolean retour;
 	gchar *new_filename;
@@ -318,7 +487,7 @@ static gboolean gsb_file_save_backup (void)
     }
     /* create a filename for the backup :
      * filename_yyyymmddTmmhhss.gsb */
-    if (conf.make_bakup_single_file)
+    if (make_bakup_single_file)
     {
         new_filename =  g_strdup_printf ("%s%s%s_backup.gsb",
 										 gsb_file_get_backup_path (),
@@ -341,7 +510,7 @@ static gboolean gsb_file_save_backup (void)
 										 day_time->tm_sec);
     }
 
-    retour = gsb_file_save_save_file (new_filename, conf.compress_backup, FALSE);
+    retour = gsb_file_save_save_file (new_filename, compress_backup, FALSE);
 
     g_free (new_filename);
     g_free (name);
@@ -359,17 +528,20 @@ static gboolean gsb_file_save_backup (void)
  *
  * \return TRUE to continue the timeout, FALSE to stop the timeout
  **/
-static gboolean gsb_file_automatic_backup (gpointer null)
+static gboolean gsb_file_automatic_backup (gpointer p_conf)
 {
-    devel_debug (NULL);
+	GrisbiAppConf *a_conf;
 
-    if (!conf.make_backup_every_minutes)
+	devel_debug (NULL);
+	a_conf = (GrisbiAppConf *) p_conf;
+
+    if (!a_conf->make_backup_every_minutes)
 	/* stop the timeout */
         return FALSE;
 
     /* we save only if there is a nb of minutes, but don't stop the timer if not */
-    if (conf.make_backup_nb_minutes)
-        gsb_file_save_backup ();
+    if (a_conf->make_backup_nb_minutes)
+        gsb_file_save_backup (a_conf->make_bakup_single_file, a_conf->compress_backup);
 
     return TRUE;
 }
@@ -386,6 +558,7 @@ static gboolean gsb_file_save_file (gint origine)
     gint result = 0;
     gchar *nouveau_nom_enregistrement;
 	gchar *filename;
+	GrisbiAppConf *a_conf;
 
     devel_debug_int (origine);
 
@@ -398,11 +571,13 @@ static gboolean gsb_file_save_file (gint origine)
         return (TRUE);
     }
 
+	a_conf = grisbi_app_get_a_conf ();
+
 	/* on récupère le nom du fichier */
 	filename = g_strdup (grisbi_win_get_filename (NULL));
 
 	/* on vérifie que le fichier n'est pas locké si il l'est on sort */
-    if (etat.fichier_deja_ouvert && !conf.force_enregistrement && origine != -2)
+    if (etat.fichier_deja_ouvert && !a_conf->force_enregistrement && origine != -2)
     {
         gchar* tmp_str1;
         gchar* tmp_str2;
@@ -420,7 +595,7 @@ static gboolean gsb_file_save_file (gint origine)
     }
 
     /* on commence par demander si on sauvegarde ou pas sauf pour "sauvegarder sous" */
-    if (!conf.sauvegarde_auto && origine != -2)
+    if (!a_conf->sauvegarde_auto && origine != -2)
     {
         result = gsb_file_dialog_save (filename, origine);
         if (result == GTK_RESPONSE_NO)
@@ -428,7 +603,7 @@ static gboolean gsb_file_save_file (gint origine)
 			gsb_file_set_modified (FALSE);
             return (TRUE);
 		}
-		else if (result == GTK_RESPONSE_REJECT)
+		else if (result != GTK_RESPONSE_OK)
 		{
 			//~ gsb_file_set_modified (FALSE);
             return (FALSE);
@@ -446,13 +621,13 @@ static gboolean gsb_file_save_file (gint origine)
         return FALSE;
 
     /* make backup before saving if asked */
-    if (conf.make_backup)
-        gsb_file_save_backup();
+    if (a_conf->sauvegarde_fermeture)
+        gsb_file_save_backup (a_conf->make_bakup_single_file, a_conf->compress_backup);
 
     /*  on a maintenant un nom de fichier et on sait qu'on peut sauvegarder */
     grisbi_win_status_bar_message (_("Saving file"));
 
-    result = gsb_file_save_save_file (nouveau_nom_enregistrement, conf.compress_file, FALSE);
+    result = gsb_file_save_save_file (nouveau_nom_enregistrement, a_conf->compress_file, FALSE);
 
     if (result)
     {
@@ -472,6 +647,19 @@ static gboolean gsb_file_save_file (gint origine)
         etat.fichier_deja_ouvert = 0;
         gsb_file_set_modified (FALSE);
         grisbi_win_set_window_title (gsb_gui_navigation_get_current_account ());
+
+		/* Si nettoyage des fichiers de backup on le fait ici */
+		if (a_conf->remove_backup_files)
+		{
+			GrisbiWinRun *w_run;
+
+			w_run = (GrisbiWinRun *) grisbi_win_get_w_run ();
+			if (!w_run->remove_backup_files)
+			{
+				gsb_file_remove_old_backup (nouveau_nom_enregistrement,a_conf->remove_backup_months);
+				w_run->remove_backup_files = TRUE;
+			}
+		}
     }
 
 	g_free (filename);
@@ -612,8 +800,6 @@ void gsb_file_init_last_path (const gchar *last_path)
  **/
 void gsb_file_update_last_path (const gchar *last_path)
 {
-    GSettings *settings;
-
     devel_debug (last_path);
 
     if (last_path && strlen (last_path))
@@ -621,8 +807,6 @@ void gsb_file_update_last_path (const gchar *last_path)
         if (last_path_used)
             g_free (last_path_used);
         last_path_used = my_strdup (last_path);
-        settings = grisbi_settings_get_settings (SETTINGS_GENERAL);
-        g_settings_set_string (G_SETTINGS (settings), "last-path", last_path_used);
     }
 }
 
@@ -685,7 +869,10 @@ void gsb_file_set_backup_path (const gchar *path)
  **/
 gboolean gsb_file_open_file (const gchar *filename)
 {
-    devel_debug (filename);
+	GrisbiAppConf *a_conf;
+
+	devel_debug (filename);
+	a_conf = grisbi_app_get_a_conf ();
 
 	if (!gsb_file_test_file (filename))
 	{
@@ -706,7 +893,6 @@ gboolean gsb_file_open_file (const gchar *filename)
 
     if (gsb_file_load_open_file (filename))
     {
-
         /* the file has been opened succesfully */
         /* on met à jour le nom du fichier */
         grisbi_win_set_filename (NULL, filename);
@@ -715,10 +901,9 @@ gboolean gsb_file_open_file (const gchar *filename)
         gsb_file_util_modify_lock (filename, TRUE);
 
         /* we make a backup if necessary */
-        if (conf.sauvegarde_demarrage)
+        if (a_conf->sauvegarde_demarrage)
         {
-            gsb_file_save_backup ();
-            gsb_file_set_modified (FALSE);
+			gsb_file_save_backup (a_conf->make_bakup_single_file, a_conf->compress_backup);
         }
     }
     else
@@ -746,7 +931,7 @@ gboolean gsb_file_open_file (const gchar *filename)
 
 		tmp_str1 = g_strdup_printf (_("Error loading file '%s'"), filename);
 
-        if (conf.sauvegarde_demarrage || conf.make_backup || conf.make_backup_every_minutes)
+        if (a_conf->sauvegarde_demarrage || a_conf->sauvegarde_fermeture || a_conf->make_backup_every_minutes)
         {
             tmp_str2 = g_strdup_printf (_("Grisbi was unable to load file. You should find the last "
 										  "backups in '%s', they are saved with date and time into "
@@ -845,7 +1030,10 @@ gboolean gsb_file_save_as (void)
 gboolean gsb_file_automatic_backup_start (GtkWidget *checkbutton,
 										  gpointer null)
 {
-    devel_debug_int (conf.make_backup_every_minutes);
+	GrisbiAppConf *a_conf;
+
+	a_conf = grisbi_app_get_a_conf ();
+	devel_debug_int (a_conf->make_backup_every_minutes);
 
     /* if there is already a timeout, we remove it */
     if (id_timeout)
@@ -855,10 +1043,10 @@ gboolean gsb_file_automatic_backup_start (GtkWidget *checkbutton,
     }
 
     /* launch the timeout only if active and if there is some minutes */
-    if (conf.make_backup_every_minutes && conf.make_backup_nb_minutes)
-		id_timeout = g_timeout_add_seconds (conf.make_backup_nb_minutes * 60,
-											(GSourceFunc) (gsb_file_automatic_backup),
-											NULL);
+    if (a_conf->make_backup_every_minutes && a_conf->make_backup_nb_minutes)
+		id_timeout = g_timeout_add_seconds (a_conf->make_backup_nb_minutes * 60,
+											gsb_file_automatic_backup,
+											a_conf);
     return FALSE;
 }
 
@@ -873,7 +1061,10 @@ gboolean gsb_file_automatic_backup_start (GtkWidget *checkbutton,
 gboolean gsb_file_automatic_backup_change_time (GtkWidget *spinbutton,
 												gpointer null)
 {
-    devel_debug_int (conf.make_backup_nb_minutes);
+	GrisbiAppConf *a_conf;
+
+	a_conf = grisbi_app_get_a_conf ();
+	devel_debug_int (a_conf->make_backup_every_minutes);
 
     /* if there is already a timeout, we stop it */
     if (id_timeout)
@@ -883,10 +1074,10 @@ gboolean gsb_file_automatic_backup_change_time (GtkWidget *spinbutton,
     }
 
     /* set a new timeout only if there is an interval */
-    if (conf.make_backup_nb_minutes)
-	id_timeout = g_timeout_add_seconds (conf.make_backup_nb_minutes * 60,
-					     (GSourceFunc) (gsb_file_automatic_backup),
-					     NULL);
+    if (a_conf->make_backup_nb_minutes)
+	id_timeout = g_timeout_add_seconds (a_conf->make_backup_nb_minutes * 60,
+										gsb_file_automatic_backup,
+										a_conf);
 
     return FALSE;
 }
@@ -902,13 +1093,16 @@ gboolean gsb_file_automatic_backup_change_time (GtkWidget *spinbutton,
 gboolean gsb_file_close (void)
 {
 	gchar *filename;
-    devel_debug (NULL);
+	GrisbiAppConf *a_conf;
 
+	devel_debug (NULL);
     if (!assert_account_loaded ())
 	{
         return TRUE;
 	}
 
+	a_conf = grisbi_app_get_a_conf ();
+
 	/* on récupère le nom du fichier */
 	filename = g_strdup (grisbi_win_get_filename (NULL));
 
@@ -918,7 +1112,7 @@ gboolean gsb_file_close (void)
 	    if (!gsb_file_save_file (-1))
             return FALSE;
     }
-    else if (conf.sauvegarde_auto && filename)
+    else if (a_conf->sauvegarde_auto && filename)
     {
         /* try to save */
 	    if (!gsb_file_save_file (-1))
@@ -975,10 +1169,8 @@ void gsb_file_set_modified (gboolean modified)
 
 	if (modified)
     {
-        if (!run.file_modification)
-        {
-            run.file_modification = time (NULL);
-        }
+		/* modification pour gerer la non modification par la recherche dans la liste des operations */
+		run.file_modification = time (NULL);
         gsb_menu_gui_sensitive_win_menu_item ("save", TRUE);
     }
     else
@@ -1080,6 +1272,71 @@ void gsb_file_copy_old_file (const gchar *filename)
 		g_object_unref (file_copy);
     }
 }
+/**
+ * Fonction de fermeture de grisbi sans appeler la fonction init_variables ().
+ * Demandera une clarification de la libération des variables.
+ *
+ * \param
+ *
+ * \return
+ **/
+gboolean gsb_file_quit (void)
+{
+	gchar *filename;
+	GrisbiAppConf *a_conf;
+
+	devel_debug (NULL);
+    if (!assert_account_loaded ())
+	{
+        return TRUE;
+	}
+
+	a_conf = grisbi_app_get_a_conf ();
+
+	/* on récupère le nom du fichier */
+	filename = g_strdup (grisbi_win_get_filename (NULL));
+
+	if (gsb_file_get_modified ())
+    {
+        /* try to save */
+	    if (!gsb_file_save_file (-1))
+            return FALSE;
+    }
+    else if (a_conf->sauvegarde_auto && filename)
+    {
+        /* try to save */
+	    if (!gsb_file_save_file (-1))
+            return FALSE;
+    }
+
+    if (!gsb_file_get_modified ())
+    {
+	     /* remove the lock */
+	    if (!etat.fichier_deja_ouvert
+			&&
+			gsb_data_account_get_accounts_amount ()
+			&&
+			filename)
+		{
+			gsb_file_util_modify_lock (filename, FALSE);
+		}
+
+	    /* free all the variables */
+		etats_gtktable_free_table_etat (); /* set table_etat = NULL: fix crash loading a multiple accounts files */
+        grisbi_win_free_general_vbox ();
+        gsb_account_property_clear_config ();
+		grisbi_win_set_filename (NULL, NULL);
+
+        grisbi_win_set_window_title (-1);
+        grisbi_win_menu_move_to_acc_delete ();
+
+		g_free (filename);
+	    return TRUE;
+    }
+
+    return FALSE;
+}
+
 /**
  *
  *
