@@ -6,7 +6,7 @@
 /*     Copyright (C)    2000-2008 Cédric Auger (cedric@grisbi.org)               */
 /*                      2003-2008 Benjamin Drieu (bdrieu@april.org)              */
 /*          2008-2018 Pierre Biava (grisbi@pierre.biava.name)                    */
-/*          http://www.grisbi.org                                                */
+/*          https://www.grisbi.org/                                               */
 /*                                                                               */
 /*     This program is free software; you can redistribute it and/or modify      */
 /*     it under the terms of the GNU General Public License as published by      */
@@ -37,6 +37,7 @@
 #include <goffice/goffice.h>
 #endif /* HAVE_GOFFICE */
 
+
 #ifdef G_OS_WIN32
 #include <stdio.h>
 #include <stdlib.h>
@@ -49,12 +50,19 @@
 #include "gsb_dirs.h"
 #include "gsb_locale.h"
 #include "structures.h"
+#include "erreur.h"
+#ifdef __APPLE__
+#   include "grisbi_osx.h"
+#endif
 /*END_INCLUDE*/
 
 /*START_EXTERN*/
 /*END_EXTERN*/
 
 
+
+
+
 /**
  * Main function
  *
@@ -67,15 +75,26 @@ int main (int argc, char **argv)
 {
     GrisbiApp *app;
 	gint status;
-
+#if defined(HAVE_GOFFICE) || defined(__APPLE__)
+    GSList *goffice_plugins_dirs = NULL;
+#endif
 	/* On force l'utilisation de X11 en attendant que grisbi fonctionne correctement sous wayland */
-#ifdef GDK_WINDOWING_WAYLAND
+/*#ifdef GDK_WINDOWING_WAYLAND
 	#ifdef GDK_WINDOWING_X11
 		gdk_set_allowed_backends ("x11");
 	#else
 		return (1);
 	#endif
 #endif
+*/
+
+#ifdef EARLY_DEBUG
+	debug_start_log_file ();
+#endif
+
+#ifdef __APPLE__
+    goffice_plugins_dirs = grisbi_osx_init(&argc, &argv);
+#endif
 
 	/* On commence par initialiser les répertoires */
     gsb_dirs_init (argv[0]);
@@ -90,7 +109,7 @@ int main (int argc, char **argv)
     /* initialisation libgoffice */
     libgoffice_init ();
     /* Initialize plugins manager */
-    go_plugins_init (NULL, NULL, NULL, NULL, TRUE, GO_TYPE_PLUGIN_LOADER_MODULE);
+    go_plugins_init (NULL, NULL, NULL, goffice_plugins_dirs, TRUE, GO_TYPE_PLUGIN_LOADER_MODULE);
 #endif /* HAVE_GOFFICE */
 
     app = g_object_new (GRISBI_APP_TYPE,
@@ -108,6 +127,7 @@ int main (int argc, char **argv)
 #ifdef HAVE_GOFFICE
     /* liberation libgoffice */
     libgoffice_shutdown ();
+    g_slist_free(goffice_plugins_dirs); //, g_free);
 #endif /* HAVE_GOFFICE */
 
     return status;
