@@ -4,7 +4,7 @@
 /*     Copyright (C)    2000-2008 Cedric Auger (cedric@grisbi.org)            */
 /*          2003-2008 Benjamin Drieu (bdrieu@april.org)                       */
 /*          2008-2018 Pierre Biava (grisbi@pierre.biava.name)                 */
-/*          http://www.grisbi.org                                             */
+/*          https://www.grisbi.org/                                            */
 /*                                                                            */
 /*  This program is free software; you can redistribute it and/or modify      */
 /*  it under the terms of the GNU General Public License as published by      */
@@ -36,6 +36,7 @@
 /*START_INCLUDE*/
 #include "utils_str.h"
 #include "dialog.h"
+#include "grisbi_app.h"
 #include "gsb_data_report.h"
 #include "gsb_data_currency.h"
 #include "gsb_locale.h"
@@ -384,18 +385,6 @@ gint utils_str_atoi (const gchar *chaine)
     }
 }
 
-/**
- *
- *
- * \param
- *
- * \return
- **/
-gchar * latin2utf8 (const gchar *inchar)
-{
-    return g_locale_from_utf8 (inchar, -1, NULL, NULL, NULL);
-}
-
 /**
  * do the same as g_strdelimit but new_delimiters can containes several characters or none
  * ex	my_strdelimit ("a-b", "-", "123") returns a123b
@@ -519,8 +508,8 @@ gint my_strcasecmp (const gchar *string_1,
  *
  * \return
  **/
-gint my_strncasecmp (gchar *string_1,
-					 gchar *string_2,
+gint my_strncasecmp (const gchar *string_1,
+					 const gchar *string_2,
 					 gint longueur)
 {
     if (!string_1 && string_2)
@@ -581,7 +570,7 @@ gchar *my_strdup (const gchar *string)
  *
  * \return a duplicate version of the string with max length character (must to be freed)
  **/
-gchar *limit_string (gchar *string,
+gchar *limit_string (const gchar *string,
 					 gint length)
 {
     gchar *string_return;
@@ -612,7 +601,7 @@ gchar *limit_string (gchar *string,
  * \return a g_slist or NULL
  **/
 GSList *gsb_string_get_int_list_from_string (const gchar *string,
-											 gchar *delimiter)
+											 const gchar *delimiter)
 {
     GSList *list_tmp;
     gchar **tab;
@@ -647,7 +636,7 @@ GSList *gsb_string_get_int_list_from_string (const gchar *string,
  * \return a g_slist or NULL
  **/
 GSList *gsb_string_get_string_list_from_string (const gchar *string,
-												gchar *delimiter)
+												const gchar *delimiter)
 {
     GSList *list_tmp;
     gchar **tab;
@@ -747,8 +736,8 @@ gchar * gsb_string_truncate (gchar * string)
  * \return
  **/
 gchar *gsb_string_remplace_string (const gchar *str,
-								   gchar *old_str,
-								   gchar *new_str)
+								   const gchar *old_str,
+								   const gchar *new_str)
 {
     gchar *ptr_debut;
     size_t long_old, str_len;
@@ -1210,6 +1199,7 @@ gchar *utils_str_protect_unprotect_multilines_text (const gchar *text,
 		if (g_strstr_len (tmp_str, -1, "\\n" ) )
 		{
 			tmp_tab = g_strsplit (tmp_str, "\\n", 0);
+			g_free (tmp_str);
 			tmp_str = g_strjoinv (NEW_LINE, tmp_tab);
 
 			g_strfreev (tmp_tab);
@@ -1244,7 +1234,11 @@ gchar *utils_str_remove_accents (const gchar *text)
 	nbre_chars = g_utf8_strlen (new_text, -1);
 
 	if (nbre_bytes == nbre_chars)
+	{
+		g_free (new_text);
+
 		return g_strdup (text);
+	}
 
     tmp_str = g_malloc0 (nbre_bytes * sizeof (gchar));
     ptr = (gchar*) new_text;
@@ -1261,6 +1255,8 @@ gchar *utils_str_remove_accents (const gchar *text)
         ptr = g_utf8_next_char (ptr);
     }
 
+	g_free (new_text);
+
 	return tmp_str;
 }
 
@@ -1297,6 +1293,182 @@ gchar *utils_str_my_case_strstr (const gchar *haystack,
 	return ptr;
 }
 
+/**
+ * Remplace un espace par un "\n" à partir de nbre de caractères
+ *
+ * \param
+ * \param
+ *
+ * \return
+ **/
+gchar *utils_str_break_form_name_field (const gchar *text,
+										gint trunc)
+{
+	gchar *tmp_str = NULL;
+	glong nbre_char;
+
+	if (!text)
+		return NULL;
+
+	if (!grisbi_app_get_low_definition_screen ())
+	{
+		tmp_str = g_strdup (text);
+		return (tmp_str);
+	}
+
+	if (g_utf8_validate (text, -1, NULL))
+	{
+		nbre_char = g_utf8_strlen (text, -1);
+		if (nbre_char <= trunc)
+			return g_strdup (text);
+
+		if (g_utf8_strchr (text, -1, ':'))
+		{
+			gchar **tab;
+
+			tab= g_strsplit (text, ":", 2);
+			tmp_str = g_strconcat (tab[0], ":\n", tab[1]+1, NULL);
+		}
+		else if (g_utf8_strchr (text, -1, '|'))
+		{
+			gchar **tab;
+
+			tab= g_strsplit (text, "||", 0);
+			tmp_str = g_strjoinv ("||\n", tab);
+			g_strfreev (tab);
+		}
+		else if (g_utf8_strchr (text, -1, ' '))
+		{
+			gchar *tmp_str2 = NULL;
+			gchar *ptr_free;
+			gchar *ptr_free2;
+			gchar **tab;
+			gint i;
+			gint nbre_champs;
+
+			tab= g_strsplit (text, " ", 0);
+			nbre_champs = g_strv_length (tab);
+
+			for (i = 0; tab[i]; i++)
+			{
+				if (i == 0)
+				{
+					tmp_str = g_strdup (tab[i]);
+					nbre_char = g_utf8_strlen (tmp_str, -1);
+					if (nbre_char > trunc-1)
+					{
+						ptr_free = tmp_str;
+						tmp_str = g_strconcat (tmp_str, "\n", NULL);
+						g_free (ptr_free);
+					}
+					else
+					{
+						if (tab[i+1])
+						{
+							nbre_char = g_utf8_strlen (tmp_str, -1) + g_utf8_strlen (tab[i+1], -1) + 1;
+							if (nbre_char < trunc)
+								i++;
+							while (nbre_char < trunc)
+							{
+								ptr_free = tmp_str;
+								if (g_utf8_strlen (tab[i], -1) == 0)
+									i++;
+								else
+								{
+									tmp_str = g_strconcat (tmp_str, " ", tab[i], NULL);
+									g_free (ptr_free);
+								}
+								if (!tab[i+1])
+									break;
+
+								nbre_char = g_utf8_strlen (tmp_str, -1) + g_utf8_strlen (tab[i+1], -1) + 1;
+								if (nbre_char < trunc)
+									i++;
+							};
+						}
+						if (i < nbre_champs-1)
+						{
+							ptr_free = tmp_str;
+							tmp_str = g_strconcat (tmp_str, "\n", NULL);
+							g_free (ptr_free);
+						}
+					}
+				}
+				else
+				{
+					tmp_str2 = g_strdup (tab[i]);
+					nbre_char = g_utf8_strlen (tmp_str2, -1);
+					if (nbre_char > trunc-1)
+					{
+						ptr_free = tmp_str;
+						ptr_free2 = tmp_str2;
+						tmp_str = g_strconcat (tmp_str, tmp_str2, "\n", NULL);
+						g_free (ptr_free);
+						g_free (ptr_free2);
+					}
+					else
+					{
+						if (tab[i+1])
+						{
+							nbre_char = g_utf8_strlen (tmp_str2, -1) + g_utf8_strlen (tab[i+1], -1) + 1;
+							if (nbre_char < trunc)
+								i++;
+							while (nbre_char < trunc)
+							{
+								ptr_free2 = tmp_str2;
+								if (g_utf8_strlen (tab[i], -1) == 0)
+									i++;
+								else
+								{
+									tmp_str2 = g_strconcat (tmp_str2, " ", tab[i], NULL);
+									g_free (ptr_free2);
+								}
+								if (!tab[i+1])
+									break;
+
+								nbre_char = g_utf8_strlen (tmp_str2, -1) + g_utf8_strlen (tab[i+1], -1) + 1;
+								if (nbre_char < trunc)
+									i++;
+							};
+						}
+						ptr_free = tmp_str;
+						ptr_free2 = tmp_str2;
+						if (i < nbre_champs-1)
+						{
+							tmp_str = g_strconcat (tmp_str, tmp_str2, "\n", NULL);
+						}
+						else
+						{
+							tmp_str = g_strconcat (tmp_str, tmp_str2, NULL);
+						}
+						g_free (ptr_free);
+						g_free (ptr_free2);
+					}
+				}
+			}
+			g_strfreev (tab);
+		}
+		else if (g_utf8_strchr (text, -1, '/'))
+		{
+			gchar *ptr;
+
+			/* on recherche le dernier slash */
+			ptr = g_utf8_strrchr (text, -1, '/');
+			ptr[0] = '\n';
+		}
+		else
+		{
+			tmp_str = g_strdup (text);
+		}
+	}
+	else
+	{
+		tmp_str = g_strdup (text);
+	}
+
+	return tmp_str;
+}
+
 /**
  *
  *
